@@ -1,12 +1,7 @@
-/* eslint-disable comma-dangle, no-alert */
-
 import $ from 'jquery';
 import Vue from 'vue';
 
-window.gl = window.gl || {};
-window.gl.issueBoards = window.gl.issueBoards || {};
-
-gl.issueBoards.BoardDelete = Vue.extend({
+export default Vue.extend({
   props: {
     list: {
       type: Object,
@@ -14,12 +9,13 @@ gl.issueBoards.BoardDelete = Vue.extend({
     },
   },
   methods: {
-    deleteBoard () {
+    deleteBoard() {
       $(this.$el).tooltip('hide');
 
+      // eslint-disable-next-line no-alert
       if (window.confirm('Are you sure you want to delete this list?')) {
         this.list.destroy();
       }
-    }
-  }
+    },
+  },
 });
