@@ -2,7 +2,7 @@ import $ from 'jquery';
 import Cookies from 'js-cookie';
 
 export default () => {
-  $('.js-experiment-feature-toggle').on('change', (e) => {
+  $('.js-experiment-feature-toggle').on('change', e => {
     const el = e.target;
 
     Cookies.set(el.name, el.value, {
