@@ -1,8 +1,5 @@
 import _ from 'underscore';
-import {
-  getParameterByName,
-  getUrlParamsArray,
-} from '~/lib/utils/common_utils';
+import { getParameterByName, getUrlParamsArray } from '~/lib/utils/common_utils';
 import IssuableFilteredSearchTokenKeys from '~/filtered_search/issuable_filtered_search_token_keys';
 import { visitUrl } from '../lib/utils/url_utility';
 import Flash from '../flash';
@@ -48,24 +45,28 @@ export default class FilteredSearchManager {
       isLocalStorageAvailable: RecentSearchesService.isAvailable(),
       allowedKeys: this.filteredSearchTokenKeys.getKeys(),
     });
-    this.searchHistoryDropdownElement = document.querySelector('.js-filtered-search-history-dropdown');
-    const fullPath = this.searchHistoryDropdownElement ?
-      this.searchHistoryDropdownElement.dataset.fullPath : 'project';
+    this.searchHistoryDropdownElement = document.querySelector(
+      '.js-filtered-search-history-dropdown',
+    );
+    const fullPath = this.searchHistoryDropdownElement
+      ? this.searchHistoryDropdownElement.dataset.fullPath
+      : 'project';
     const recentSearchesKey = `${fullPath}-${this.recentsStorageKeyNames[this.page]}`;
     this.recentSearchesService = new RecentSearchesService(recentSearchesKey);
   }
 
   setup() {
     // Fetch recent searches from localStorage
-    this.fetchingRecentSearchesPromise = this.recentSearchesService.fetch()
-      .catch((error) => {
+    this.fetchingRecentSearchesPromise = this.recentSearchesService
+      .fetch()
+      .catch(error => {
         if (error.name === 'RecentSearchesServiceError') return undefined;
         // eslint-disable-next-line no-new
         new Flash('An error occurred while parsing recent searches');
         // Gracefully fail to empty array
         return [];
       })
-      .then((searches) => {
+      .then(searches => {
         if (!searches) {
           return;
         }
@@ -120,7 +121,7 @@ export default class FilteredSearchManager {
     if (this.stateFilters) {
       this.searchStateWrapper = this.searchState.bind(this);
 
-      this.applyToStateFilters((filterEl) => {
+      this.applyToStateFilters(filterEl => {
         filterEl.addEventListener('click', this.searchStateWrapper);
       });
     }
@@ -128,14 +129,14 @@ export default class FilteredSearchManager {
 
   unbindStateEvents() {
     if (this.stateFilters) {
-      this.applyToStateFilters((filterEl) => {
+      this.applyToStateFilters(filterEl => {
         filterEl.removeEventListener('click', this.searchStateWrapper);
       });
     }
   }
 
   applyToStateFilters(callback) {
-    this.stateFilters.querySelectorAll('a[data-state]').forEach((filterEl) => {
+    this.stateFilters.querySelectorAll('a[data-state]').forEach(filterEl => {
       if (this.states.indexOf(filterEl.dataset.state) > -1) {
         callback(filterEl);
       }
@@ -207,7 +208,7 @@ export default class FilteredSearchManager {
     let backspaceCount = 0;
 
     // closure for keeping track of the number of backspace keystrokes
-    return (e) => {
+    return e => {
       // 8 = Backspace Key
       // 46 = Delete Key
       if (e.keyCode === 8 || e.keyCode === 46) {
@@ -274,8 +275,12 @@ export default class FilteredSearchManager {
     const isElementInDynamicFilterDropdown = e.target.closest('.filter-dropdown') !== null;
     const isElementInStaticFilterDropdown = e.target.closest('ul[data-dropdown]') !== null;
 
-    if (!isElementInFilteredSearch && !isElementInDynamicFilterDropdown &&
-      !isElementInStaticFilterDropdown && inputContainer) {
+    if (
+      !isElementInFilteredSearch &&
+      !isElementInDynamicFilterDropdown &&
+      !isElementInStaticFilterDropdown &&
+      inputContainer
+    ) {
       inputContainer.classList.remove('focus');
     }
   }
@@ -368,7 +373,7 @@ export default class FilteredSearchManager {
 
     const removeElements = [];
 
-    [].forEach.call(this.tokensContainer.children, (t) => {
+    [].forEach.call(this.tokensContainer.children, t => {
       let canClearToken = t.classList.contains('js-visual-token');
 
       if (canClearToken) {
@@ -381,7 +386,7 @@ export default class FilteredSearchManager {
       }
     });
 
-    removeElements.forEach((el) => {
+    removeElements.forEach(el => {
       el.parentElement.removeChild(el);
     });
 
@@ -397,13 +402,14 @@ export default class FilteredSearchManager {
 
   handleInputVisualToken() {
     const input = this.filteredSearchInput;
-    const { tokens, searchToken }
-      = this.tokenizer.processTokens(input.value, this.filteredSearchTokenKeys.getKeys());
-    const { isLastVisualTokenValid }
-      = FilteredSearchVisualTokens.getLastVisualTokenBeforeInput();
+    const { tokens, searchToken } = this.tokenizer.processTokens(
+      input.value,
+      this.filteredSearchTokenKeys.getKeys(),
+    );
+    const { isLastVisualTokenValid } = FilteredSearchVisualTokens.getLastVisualTokenBeforeInput();
 
     if (isLastVisualTokenValid) {
-      tokens.forEach((t) => {
+      tokens.forEach(t => {
         input.value = input.value.replace(`${t.key}:${t.symbol}${t.value}`, '');
         FilteredSearchVisualTokens.addFilterVisualToken(t.key, `${t.symbol}${t.value}`, {
           uppercaseTokenName: this.filteredSearchTokenKeys.shouldUppercaseTokenName(t.key),
@@ -453,15 +459,17 @@ export default class FilteredSearchManager {
 
   saveCurrentSearchQuery() {
     // Don't save before we have fetched the already saved searches
-    this.fetchingRecentSearchesPromise.then(() => {
-      const searchQuery = DropdownUtils.getSearchQuery();
-      if (searchQuery.length > 0) {
-        const resultantSearches = this.recentSearchesStore.addRecentSearch(searchQuery);
-        this.recentSearchesService.save(resultantSearches);
-      }
-    }).catch(() => {
-      // https://gitlab.com/gitlab-org/gitlab-ce/issues/30821
-    });
+    this.fetchingRecentSearchesPromise
+      .then(() => {
+        const searchQuery = DropdownUtils.getSearchQuery();
+        if (searchQuery.length > 0) {
+          const resultantSearches = this.recentSearchesStore.addRecentSearch(searchQuery);
+          this.recentSearchesService.save(resultantSearches);
+        }
+      })
+      .catch(() => {
+        // https://gitlab.com/gitlab-org/gitlab-ce/issues/30821
+      });
   }
 
   // allows for modifying params array when a param can't be included in the URL (e.g. Service Desk)
@@ -475,7 +483,7 @@ export default class FilteredSearchManager {
     const usernameParams = this.getUsernameParams();
     let hasFilteredSearch = false;
 
-    params.forEach((p) => {
+    params.forEach(p => {
       const split = p.split('=');
       const keyParam = decodeURIComponent(split[0]);
       const value = split[1];
@@ -486,11 +494,9 @@ export default class FilteredSearchManager {
       if (condition) {
         hasFilteredSearch = true;
         const canEdit = this.canEdit && this.canEdit(condition.tokenKey);
-        FilteredSearchVisualTokens.addFilterVisualToken(
-          condition.tokenKey,
-          condition.value,
-          { canEdit },
-        );
+        FilteredSearchVisualTokens.addFilterVisualToken(condition.tokenKey, condition.value, {
+          canEdit,
+        });
       } else {
         // Sanitize value since URL converts spaces into +
         // Replace before decode so that we know what was originally + versus the encoded +
@@ -510,7 +516,7 @@ export default class FilteredSearchManager {
 
           if (sanitizedValue.indexOf(' ') !== -1) {
             // Prefer ", but use ' if required
-            quotationsToUse = sanitizedValue.indexOf('"') === -1 ? '"' : '\'';
+            quotationsToUse = sanitizedValue.indexOf('"') === -1 ? '"' : "'";
           }
 
           hasFilteredSearch = true;
@@ -531,7 +537,9 @@ export default class FilteredSearchManager {
             hasFilteredSearch = true;
             const tokenName = 'assignee';
             const canEdit = this.canEdit && this.canEdit(tokenName);
-            FilteredSearchVisualTokens.addFilterVisualToken(tokenName, `@${usernameParams[id]}`, { canEdit });
+            FilteredSearchVisualTokens.addFilterVisualToken(tokenName, `@${usernameParams[id]}`, {
+              canEdit,
+            });
           }
         } else if (!match && keyParam === 'author_id') {
           const id = parseInt(value, 10);
@@ -539,7 +547,9 @@ export default class FilteredSearchManager {
             hasFilteredSearch = true;
             const tokenName = 'author';
             const canEdit = this.canEdit && this.canEdit(tokenName);
-            FilteredSearchVisualTokens.addFilterVisualToken(tokenName, `@${usernameParams[id]}`, { canEdit });
+            FilteredSearchVisualTokens.addFilterVisualToken(tokenName, `@${usernameParams[id]}`, {
+              canEdit,
+            });
           }
         } else if (!match && keyParam === 'search') {
           hasFilteredSearch = true;
@@ -580,9 +590,11 @@ export default class FilteredSearchManager {
     const currentState = state || getParameterByName('state') || 'opened';
     paths.push(`state=${currentState}`);
 
-    tokens.forEach((token) => {
-      const condition = this.filteredSearchTokenKeys
-        .searchByConditionKeyValue(token.key, token.value.toLowerCase());
+    tokens.forEach(token => {
+      const condition = this.filteredSearchTokenKeys.searchByConditionKeyValue(
+        token.key,
+        token.value,
+      );
       const tokenConfig = this.filteredSearchTokenKeys.searchByKey(token.key) || {};
       const { param } = tokenConfig;
 
@@ -601,8 +613,10 @@ export default class FilteredSearchManager {
           tokenValue = tokenValue.toLowerCase();
         }
 
-        if ((tokenValue[0] === '\'' && tokenValue[tokenValue.length - 1] === '\'') ||
-          (tokenValue[0] === '"' && tokenValue[tokenValue.length - 1] === '"')) {
+        if (
+          (tokenValue[0] === "'" && tokenValue[tokenValue.length - 1] === "'") ||
+          (tokenValue[0] === '"' && tokenValue[tokenValue.length - 1] === '"')
+        ) {
           tokenValue = tokenValue.slice(1, tokenValue.length - 1);
         }
 
@@ -613,7 +627,10 @@ export default class FilteredSearchManager {
     });
 
     if (searchToken) {
-      const sanitized = searchToken.split(' ').map(t => encodeURIComponent(t)).join('+');
+      const sanitized = searchToken
+        .split(' ')
+        .map(t => encodeURIComponent(t))
+        .join('+');
       paths.push(`search=${sanitized}`);
     }
 
@@ -630,7 +647,7 @@ export default class FilteredSearchManager {
     const usernamesById = {};
     try {
       const attribute = this.filteredSearchInput.getAttribute('data-username-params');
-      JSON.parse(attribute).forEach((user) => {
+      JSON.parse(attribute).forEach(user => {
         usernamesById[user.id] = user.username;
       });
     } catch (e) {
