@@ -1,4 +1,5 @@
 import AjaxCache from '~/lib/utils/ajax_cache';
+import { trimFirstCharOfLineContent } from '~/diffs/store/utils';
 
 const REGEX_QUICK_ACTIONS = /^\/\w+.*$/gm;
 
@@ -25,18 +26,9 @@ export const getQuickActionText = note => {
   return text;
 };
 
-export const reduceDiscussionsToLineCodes = selectedDiscussions =>
-  selectedDiscussions.reduce((acc, note) => {
-    if (note.diff_discussion && note.line_code) {
-      // For context about line notes: there might be multiple notes with the same line code
-      const items = acc[note.line_code] || [];
-      items.push(note);
-
-      Object.assign(acc, { [note.line_code]: items });
-    }
-    return acc;
-  }, {});
-
 export const hasQuickActions = note => REGEX_QUICK_ACTIONS.test(note);
 
 export const stripQuickActions = note => note.replace(REGEX_QUICK_ACTIONS, '').trim();
+
+export const prepareDiffLines = diffLines =>
+  diffLines.map(line => ({ ...trimFirstCharOfLineContent(line) }));
