@@ -13,20 +13,20 @@ export default function deviseState(data) {
     return stateKey.conflicts;
   } else if (data.work_in_progress) {
     return stateKey.workInProgress;
+  } else if (this.shouldBeRebased) {
+    return stateKey.rebase;
   } else if (this.onlyAllowMergeIfPipelineSucceeds && this.isPipelineFailed) {
     return stateKey.pipelineFailed;
   } else if (this.hasMergeableDiscussionsState) {
     return stateKey.unresolvedDiscussions;
   } else if (this.isPipelineBlocked) {
     return stateKey.pipelineBlocked;
-  } else if (this.hasSHAChanged) {
+  } else if (this.isSHAMismatch) {
     return stateKey.shaMismatch;
   } else if (this.mergeWhenPipelineSucceeds) {
     return this.mergeError ? stateKey.autoMergeFailed : stateKey.mergeWhenPipelineSucceeds;
   } else if (!this.canMerge) {
     return stateKey.notAllowedToMerge;
-  } else if (this.shouldBeRebased) {
-    return stateKey.rebase;
   } else if (this.canBeMerged) {
     return stateKey.readyToMerge;
   }
