@@ -22,6 +22,14 @@ module QA
           element :issuable_dropdown_menu_milestone
         end
 
+        view 'app/views/shared/issuable/_label_dropdown.html.haml' do
+          element :issuable_label
+        end
+
+        view 'app/views/shared/issuable/form/_metadata_merge_request_assignee.html.haml' do
+          element :assign_to_me_link
+        end
+
         def create_merge_request
           click_element :issuable_create_button
         end
@@ -40,6 +48,16 @@ module QA
             click_on milestone.title
           end
         end
+
+        def select_label(label)
+          click_element :issuable_label
+
+          click_link label.title
+        end
+
+        def assign_to_me
+          click_element :assign_to_me_link
+        end
       end
     end
   end
