@@ -1,7 +1,7 @@
 module QA
-  context :manage, :smoke do
+  context 'Manage', :smoke do
     describe 'basic user login' do
-      it 'user logs in using basic credentials' do
+      it 'user logs in using basic credentials and logs out' do
         Runtime::Browser.visit(:gitlab, Page::Main::Login)
         Page::Main::Login.act { sign_in_using_credentials }
 
@@ -11,6 +11,15 @@ module QA
         Page::Main::Menu.perform do |menu|
           expect(menu).to have_personal_area
         end
+
+        Page::Main::Menu.perform do |menu|
+          menu.sign_out
+          expect(menu).not_to have_personal_area
+        end
+
+        Page::Main::Login.perform do |form|
+          expect(form.sign_in_tab?).to be(true)
+        end
       end
     end
   end
