@@ -1,25 +1,26 @@
 # frozen_string_literal: true
 
 module QA
-  context :create do
+  context 'Create' do
     describe 'Merge request rebasing' do
-      it 'user rebases source branch of merge request'  do
+      it 'user rebases source branch of merge request' do
         Runtime::Browser.visit(:gitlab, Page::Main::Login)
-        Page::Main::Login.act { sign_in_using_credentials }
+        Page::Main::Login.perform(&:sign_in_using_credentials)
 
-        project = Factory::Resource::Project.fabricate! do |project|
+        project = Resource::Project.fabricate! do |project|
           project.name = "only-fast-forward"
         end
+        project.visit!
 
-        Page::Project::Menu.act { go_to_settings }
-        Page::Project::Settings::MergeRequest.act { enable_ff_only }
+        Page::Project::Menu.perform(&:go_to_settings)
+        Page::Project::Settings::MergeRequest.perform(&:enable_ff_only)
 
-        merge_request = Factory::Resource::MergeRequest.fabricate! do |merge_request|
+        merge_request = Resource::MergeRequest.fabricate! do |merge_request|
           merge_request.project = project
           merge_request.title = 'Needs rebasing'
         end
 
-        Factory::Repository::ProjectPush.fabricate! do |push|
+        Resource::Repository::ProjectPush.fabricate! do |push|
           push.project = project
           push.file_name = "other.txt"
           push.file_content = "New file added!"
@@ -37,7 +38,7 @@ module QA
           merge_request.rebase!
 
           expect(merge_request).to have_merge_button
-          expect(merge_request.fast_forward_possible?).to be_truthy
+          expect(merge_request).to be_fast_forward_possible
         end
       end
     end
