@@ -1,15 +1,15 @@
 # frozen_string_literal: true
 
 module QA
-  context :create do
+  context 'Create' do
     describe 'SSH keys support' do
       let(:key_title) { "key for ssh tests #{Time.now.to_f}" }
 
-      it 'user adds and then removes an SSH key' do
+      it 'user adds and then removes an SSH key', :smoke do
         Runtime::Browser.visit(:gitlab, Page::Main::Login)
         Page::Main::Login.act { sign_in_using_credentials }
 
-        key = Factory::Resource::SSHKey.fabricate! do |resource|
+        key = Resource::SSHKey.fabricate! do |resource|
           resource.title = key_title
         end
 
