@@ -1,18 +1,18 @@
 # frozen_string_literal: true
 
 module QA
-  context :create do
-    describe 'Git push over HTTP', :ldap do
-      it 'user pushes code to the repository'  do
+  context 'Create' do
+    describe 'Git push over HTTP', :ldap_no_tls do
+      it 'user pushes code to the repository' do
         Runtime::Browser.visit(:gitlab, Page::Main::Login)
         Page::Main::Login.act { sign_in_using_credentials }
 
-        Factory::Repository::ProjectPush.fabricate! do |push|
+        project_push = Resource::Repository::ProjectPush.fabricate! do |push|
           push.file_name = 'README.md'
           push.file_content = '# This is a test project'
           push.commit_message = 'Add README.md'
         end
-
+        project_push.project.visit!
         Page::Project::Show.act { wait_for_push }
 
         expect(page).to have_content('README.md')
