@@ -1,7 +1,7 @@
 # frozen_string_literal: true
 
 module QA
-  context :create do
+  context 'Create' do
     describe 'SSH key support' do
       # Note: If you run this test against GDK make sure you've enabled sshd
       # See: https://gitlab.com/gitlab-org/gitlab-qa/blob/master/docs/run_qa_against_gdk.md
@@ -12,17 +12,18 @@ module QA
         Runtime::Browser.visit(:gitlab, Page::Main::Login)
         Page::Main::Login.act { sign_in_using_credentials }
 
-        key = Factory::Resource::SSHKey.fabricate! do |resource|
+        key = Resource::SSHKey.fabricate! do |resource|
           resource.title = key_title
         end
 
-        Factory::Repository::ProjectPush.fabricate! do |push|
+        project_push = Resource::Repository::ProjectPush.fabricate! do |push|
           push.ssh_key = key
           push.file_name = 'README.md'
           push.file_content = '# Test Use SSH Key'
           push.commit_message = 'Add README.md'
         end
 
+        project_push.project.visit!
         Page::Project::Show.act { wait_for_push }
 
         expect(page).to have_content('README.md')
