@@ -19,7 +19,7 @@ module Support
       allow(ENV).to receive(:[]).with(key).and_return(value)
       allow(ENV).to receive(:key?).with(key).and_return(true)
       allow(ENV).to receive(:fetch).with(key).and_return(value)
-      allow(ENV).to receive(:fetch).with(key, anything()) do |_, default_val|
+      allow(ENV).to receive(:fetch).with(key, anything) do |_, default_val|
         value || default_val
       end
     end
