@@ -114,7 +114,7 @@ describe ApplicationController do
       skip_before_action :authenticate_user!, only: :index
 
       def index
-        render text: 'authenticated'
+        render html: 'authenticated'
       end
     end
 
@@ -137,30 +137,6 @@ describe ApplicationController do
     end
   end
 
-  describe 'rescue from Gitlab::Git::Storage::Inaccessible' do
-    controller(described_class) do
-      def index
-        raise Gitlab::Git::Storage::Inaccessible.new('broken', 100)
-      end
-    end
-
-    it 'renders a 503 when storage is not available' do
-      sign_in(create(:user))
-
-      get :index
-
-      expect(response.status).to eq(503)
-    end
-
-    it 'renders includes a Retry-After header' do
-      sign_in(create(:user))
-
-      get :index
-
-      expect(response.headers['Retry-After']).to eq(100)
-    end
-  end
-
   describe 'response format' do
     controller(described_class) do
       def index
@@ -425,7 +401,7 @@ describe ApplicationController do
   context 'terms' do
     controller(described_class) do
       def index
-        render text: 'authenticated'
+        render html: 'authenticated'
       end
     end
 
@@ -447,7 +423,7 @@ describe ApplicationController do
         enforce_terms
       end
 
-      it 'redirects if the user did not accept the terms'  do
+      it 'redirects if the user did not accept the terms' do
         get :index
 
         expect(response).to have_gitlab_http_status(302)
@@ -468,7 +444,7 @@ describe ApplicationController do
       attr_reader :last_payload
 
       def index
-        render text: 'authenticated'
+        render html: 'authenticated'
       end
 
       def append_info_to_payload(payload)
@@ -484,6 +460,14 @@ describe ApplicationController do
       expect(controller.last_payload.has_key?(:response)).to be_falsey
     end
 
+    it 'does log correlation id' do
+      Gitlab::CorrelationId.use_id('new-id') do
+        get :index
+      end
+
+      expect(controller.last_payload).to include('correlation_id' => 'new-id')
+    end
+
     context '422 errors' do
       it 'logs a response with a string' do
         response = spy(ActionDispatch::Response, status: 422, body: 'Hello world', content_type: 'application/json', cookies: {})
@@ -523,7 +507,7 @@ describe ApplicationController do
   describe '#access_denied' do
     controller(described_class) do
       def index
-        access_denied!(params[:message])
+        access_denied!(params[:message], params[:status])
       end
     end
 
@@ -535,12 +519,20 @@ describe ApplicationController do
       get :index
 
       expect(response).to have_gitlab_http_status(404)
+      expect(response).to render_template('errors/not_found')
     end
 
     it 'renders a 403 when a message is passed to access denied' do
-      get :index, message: 'None shall pass'
+      get :index, params: { message: 'None shall pass' }
 
       expect(response).to have_gitlab_http_status(403)
+      expect(response).to render_template('errors/access_denied')
+    end
+
+    it 'renders a status passed to access denied' do
+      get :index, params: { status: 401 }
+
+      expect(response).to have_gitlab_http_status(401)
     end
   end
 
@@ -558,22 +550,18 @@ describe ApplicationController do
     end
 
     context 'html' do
-      it 'renders 412' do
-        get :index, text: "hi \255"
+      subject { get :index, params: { text: "hi \255" } }
 
-        expect(response).to have_gitlab_http_status(412)
-        expect(response).to render_template :precondition_failed
+      it 'renders 412' do
+        expect { subject }.to raise_error(ActionController::BadRequest)
       end
     end
 
     context 'js' do
+      subject { get :index, format: :js, params: { text: "hi \255" } }
+
       it 'renders 412' do
-        get :index, text: "hi \255", format: :js
-
-        json_response = JSON.parse(response.body)
-
-        expect(response).to have_gitlab_http_status(412)
-        expect(json_response['error']).to eq('Invalid UTF-8')
+        expect { subject }.to raise_error(ActionController::BadRequest)
       end
     end
   end
