@@ -1,20 +1,27 @@
+# frozen_string_literal: true
+
 require 'spec_helper'
 
 describe Projects::ClustersController do
   include AccessMatchersForController
   include GoogleApi::CloudPlatformHelpers
+  include KubernetesHelpers
 
   set(:project) { create(:project) }
 
+  let(:user) { create(:user) }
+
+  before do
+    project.add_maintainer(user)
+    sign_in(user)
+  end
+
   describe 'GET index' do
+    def go(params = {})
+      get :index, params: params.reverse_merge(namespace_id: project.namespace.to_param, project_id: project)
+    end
+
     describe 'functionality' do
-      let(:user) { create(:user) }
-
-      before do
-        project.add_maintainer(user)
-        sign_in(user)
-      end
-
       context 'when project has one or more clusters' do
         let(:project) { create(:project) }
         let!(:enabled_cluster) { create(:cluster, :provided_by_gcp, projects: [project]) }
@@ -33,10 +40,11 @@ describe Projects::ClustersController do
           before do
             allow(Clusters::Cluster).to receive(:paginates_per).and_return(1)
             create_list(:cluster, 2, :provided_by_gcp, :production_environment, projects: [project])
-            get :index, namespace_id: project.namespace, project_id: project, page: last_page
           end
 
           it 'redirects to the page' do
+            go(page: last_page)
+
             expect(response).to have_gitlab_http_status(:ok)
             expect(assigns(:clusters).current_page).to eq(last_page)
           end
@@ -68,21 +76,14 @@ describe Projects::ClustersController do
       it { expect { go }.to be_denied_for(:user) }
       it { expect { go }.to be_denied_for(:external) }
     end
-
-    def go
-      get :index, namespace_id: project.namespace.to_param, project_id: project
-    end
   end
 
   describe 'GET new' do
+    def go
+      get :new, params: { namespace_id: project.namespace, project_id: project }
+    end
+
     describe 'functionality for new cluster' do
-      let(:user) { create(:user) }
-
-      before do
-        project.add_maintainer(user)
-        sign_in(user)
-      end
-
       context 'when omniauth has been configured' do
         let(:key) { 'secret-key' }
         let(:session_key_for_redirect_uri) do
@@ -121,7 +122,7 @@ describe Projects::ClustersController do
         it 'has new object' do
           go
 
-          expect(assigns(:gcp_cluster)).to be_an_instance_of(Clusters::Cluster)
+          expect(assigns(:gcp_cluster)).to be_an_instance_of(Clusters::ClusterPresenter)
         end
       end
 
@@ -139,17 +140,10 @@ describe Projects::ClustersController do
     end
 
     describe 'functionality for existing cluster' do
-      let(:user) { create(:user) }
-
-      before do
-        project.add_maintainer(user)
-        sign_in(user)
-      end
-
       it 'has new object' do
         go
 
-        expect(assigns(:user_cluster)).to be_an_instance_of(Clusters::Cluster)
+        expect(assigns(:user_cluster)).to be_an_instance_of(Clusters::ClusterPresenter)
       end
     end
 
@@ -163,10 +157,6 @@ describe Projects::ClustersController do
       it { expect { go }.to be_denied_for(:user) }
       it { expect { go }.to be_denied_for(:external) }
     end
-
-    def go
-      get :new, namespace_id: project.namespace, project_id: project
-    end
   end
 
   describe 'POST create for new cluster' do
@@ -183,14 +173,11 @@ describe Projects::ClustersController do
       }
     end
 
+    def go
+      post :create_gcp, params: params.merge(namespace_id: project.namespace, project_id: project)
+    end
+
     describe 'functionality' do
-      let(:user) { create(:user) }
-
-      before do
-        project.add_maintainer(user)
-        sign_in(user)
-      end
-
       context 'when access token is valid' do
         before do
           stub_google_api_validate_token
@@ -234,9 +221,9 @@ describe Projects::ClustersController do
     describe 'security' do
       before do
         allow_any_instance_of(described_class)
-        .to receive(:token_in_session).and_return('token')
+          .to receive(:token_in_session).and_return('token')
         allow_any_instance_of(described_class)
-        .to receive(:expires_at_in_session).and_return(1.hour.since.to_i.to_s)
+          .to receive(:expires_at_in_session).and_return(1.hour.since.to_i.to_s)
         allow_any_instance_of(GoogleApi::CloudPlatform::Client)
           .to receive(:projects_zones_clusters_create) do
           OpenStruct.new(
@@ -257,10 +244,6 @@ describe Projects::ClustersController do
       it { expect { go }.to be_denied_for(:user) }
       it { expect { go }.to be_denied_for(:external) }
     end
-
-    def go
-      post :create_gcp, params.merge(namespace_id: project.namespace, project_id: project)
-    end
   end
 
   describe 'POST create for existing cluster' do
@@ -277,14 +260,11 @@ describe Projects::ClustersController do
       }
     end
 
+    def go
+      post :create_user, params: params.merge(namespace_id: project.namespace, project_id: project)
+    end
+
     describe 'functionality' do
-      let(:user) { create(:user) }
-
-      before do
-        project.add_maintainer(user)
-        sign_in(user)
-      end
-
       context 'when creates a cluster' do
         it 'creates a new cluster' do
           expect(ClusterProvisionWorker).to receive(:perform_async)
@@ -330,6 +310,11 @@ describe Projects::ClustersController do
     end
 
     describe 'security' do
+      before do
+        allow(ClusterConfigureWorker).to receive(:perform_async)
+        stub_kubeclient_get_namespace('https://kubernetes.example.com', namespace: 'my-namespace')
+      end
+
       it { expect { go }.to be_allowed_for(:admin) }
       it { expect { go }.to be_allowed_for(:owner).of(project) }
       it { expect { go }.to be_allowed_for(:maintainer).of(project) }
@@ -339,23 +324,22 @@ describe Projects::ClustersController do
       it { expect { go }.to be_denied_for(:user) }
       it { expect { go }.to be_denied_for(:external) }
     end
-
-    def go
-      post :create_user, params.merge(namespace_id: project.namespace, project_id: project)
-    end
   end
 
-  describe 'GET status' do
+  describe 'GET cluster_status' do
     let(:cluster) { create(:cluster, :providing_by_gcp, projects: [project]) }
 
+    def go
+      get :cluster_status,
+        params: {
+          namespace_id: project.namespace.to_param,
+          project_id: project.to_param,
+          id: cluster
+        },
+        format: :json
+    end
+
     describe 'functionality' do
-      let(:user) { create(:user) }
-
-      before do
-        project.add_maintainer(user)
-        sign_in(user)
-      end
-
       it "responds with matching schema" do
         go
 
@@ -380,26 +364,21 @@ describe Projects::ClustersController do
       it { expect { go }.to be_denied_for(:user) }
       it { expect { go }.to be_denied_for(:external) }
     end
-
-    def go
-      get :status, namespace_id: project.namespace,
-                   project_id: project,
-                   id: cluster,
-                   format: :json
-    end
   end
 
   describe 'GET show' do
     let(:cluster) { create(:cluster, :provided_by_gcp, projects: [project]) }
 
+    def go
+      get :show,
+        params: {
+          namespace_id: project.namespace,
+          project_id: project,
+          id: cluster
+        }
+    end
+
     describe 'functionality' do
-      let(:user) { create(:user) }
-
-      before do
-        project.add_maintainer(user)
-        sign_in(user)
-      end
-
       it "renders view" do
         go
 
@@ -418,124 +397,50 @@ describe Projects::ClustersController do
       it { expect { go }.to be_denied_for(:user) }
       it { expect { go }.to be_denied_for(:external) }
     end
-
-    def go
-      get :show, namespace_id: project.namespace,
-                 project_id: project,
-                 id: cluster
-    end
   end
 
   describe 'PUT update' do
-    context 'when cluster is provided by GCP' do
-      let(:cluster) { create(:cluster, :provided_by_gcp, projects: [project]) }
-      let(:user) { create(:user) }
-
-      before do
-        project.add_maintainer(user)
-        sign_in(user)
-      end
-
-      context 'when changing parameters' do
-        let(:params) do
-          {
-            cluster: {
-              enabled: false,
-              name: 'my-new-cluster-name',
-              platform_kubernetes_attributes: {
-                namespace: 'my-namespace'
-              }
-            }
-          }
-        end
-
-        it "updates and redirects back to show page" do
-          go
-
-          cluster.reload
-          expect(response).to redirect_to(project_cluster_path(project, cluster))
-          expect(flash[:notice]).to eq('Kubernetes cluster was successfully updated.')
-          expect(cluster.enabled).to be_falsey
-        end
-
-        it "does not change cluster name" do
-          go
-
-          expect(cluster.name).to eq('test-cluster')
-        end
-
-        context 'when cluster is being created' do
-          let(:cluster) { create(:cluster, :providing_by_gcp, projects: [project]) }
-
-          it "rejects changes" do
-            go
-
-            expect(response).to have_gitlab_http_status(:ok)
-            expect(response).to render_template(:show)
-            expect(cluster.enabled).to be_truthy
-          end
-        end
-      end
+    def go(format: :html)
+      put :update, params: params.merge(namespace_id: project.namespace.to_param,
+                                        project_id: project.to_param,
+                                        id: cluster,
+                                        format: format
+                                       )
     end
 
-    context 'when cluster is provided by user' do
-      let(:cluster) { create(:cluster, :provided_by_user, projects: [project]) }
-      let(:user) { create(:user) }
+    before do
+      allow(ClusterConfigureWorker).to receive(:perform_async)
+      stub_kubeclient_get_namespace('https://kubernetes.example.com', namespace: 'my-namespace')
+    end
 
-      before do
-        project.add_maintainer(user)
-        sign_in(user)
-      end
+    let(:cluster) { create(:cluster, :provided_by_user, projects: [project]) }
 
-      context 'when format is json' do
-        context 'when changing parameters' do
-          context 'when valid parameters are used' do
-            let(:params) do
-              {
-                cluster: {
-                  enabled: false,
-                  name: 'my-new-cluster-name',
-                  platform_kubernetes_attributes: {
-                    namespace: 'my-namespace'
-                  }
-                }
-              }
-            end
+    let(:params) do
+      {
+        cluster: {
+          enabled: false,
+          name: 'my-new-cluster-name',
+          platform_kubernetes_attributes: {
+            namespace: 'my-namespace'
+          }
+        }
+      }
+    end
 
-            it "updates and redirects back to show page" do
-              go_json
+    it "updates and redirects back to show page" do
+      go
 
-              cluster.reload
-              expect(response).to have_http_status(:no_content)
-              expect(cluster.enabled).to be_falsey
-              expect(cluster.name).to eq('my-new-cluster-name')
-              expect(cluster.platform_kubernetes.namespace).to eq('my-namespace')
-            end
-          end
+      cluster.reload
+      expect(response).to redirect_to(project_cluster_path(project, cluster))
+      expect(flash[:notice]).to eq('Kubernetes cluster was successfully updated.')
+      expect(cluster.enabled).to be_falsey
+      expect(cluster.name).to eq('my-new-cluster-name')
+      expect(cluster.platform_kubernetes.namespace).to eq('my-namespace')
+    end
 
-          context 'when invalid parameters are used' do
-            let(:params) do
-              {
-                cluster: {
-                  enabled: false,
-                  platform_kubernetes_attributes: {
-                    namespace: 'my invalid namespace #@'
-                  }
-                }
-              }
-            end
-
-            it "rejects changes" do
-              go_json
-
-              expect(response).to have_http_status(:bad_request)
-            end
-          end
-        end
-      end
-
-      context 'when format is html' do
-        context 'when update enabled' do
+    context 'when format is json' do
+      context 'when changing parameters' do
+        context 'when valid parameters are used' do
           let(:params) do
             {
               cluster: {
@@ -549,26 +454,40 @@ describe Projects::ClustersController do
           end
 
           it "updates and redirects back to show page" do
-            go
+            go(format: :json)
 
             cluster.reload
-            expect(response).to redirect_to(project_cluster_path(project, cluster))
-            expect(flash[:notice]).to eq('Kubernetes cluster was successfully updated.')
+            expect(response).to have_http_status(:no_content)
             expect(cluster.enabled).to be_falsey
             expect(cluster.name).to eq('my-new-cluster-name')
             expect(cluster.platform_kubernetes.namespace).to eq('my-namespace')
           end
         end
+
+        context 'when invalid parameters are used' do
+          let(:params) do
+            {
+              cluster: {
+                enabled: false,
+                platform_kubernetes_attributes: {
+                  namespace: 'my invalid namespace #@'
+                }
+              }
+            }
+          end
+
+          it "rejects changes" do
+            go(format: :json)
+
+            expect(response).to have_http_status(:bad_request)
+          end
+        end
       end
     end
 
     describe 'security' do
       set(:cluster) { create(:cluster, :provided_by_gcp, projects: [project]) }
 
-      let(:params) do
-        { cluster: { enabled: false } }
-      end
-
       it { expect { go }.to be_allowed_for(:admin) }
       it { expect { go }.to be_allowed_for(:owner).of(project) }
       it { expect { go }.to be_allowed_for(:maintainer).of(project) }
@@ -578,34 +497,23 @@ describe Projects::ClustersController do
       it { expect { go }.to be_denied_for(:user) }
       it { expect { go }.to be_denied_for(:external) }
     end
-
-    def go
-      put :update, params.merge(namespace_id: project.namespace,
-                                project_id: project,
-                                id: cluster)
-    end
-
-    def go_json
-      put :update, params.merge(namespace_id: project.namespace,
-                                project_id: project,
-                                id: cluster,
-                                format: :json)
-    end
   end
 
   describe 'DELETE destroy' do
+    let!(:cluster) { create(:cluster, :provided_by_gcp, :production_environment, projects: [project]) }
+
+    def go
+      delete :destroy,
+        params: {
+          namespace_id: project.namespace,
+          project_id: project,
+          id: cluster
+        }
+    end
+
     describe 'functionality' do
-      let(:user) { create(:user) }
-
-      before do
-        project.add_maintainer(user)
-        sign_in(user)
-      end
-
       context 'when cluster is provided by GCP' do
         context 'when cluster is created' do
-          let!(:cluster) { create(:cluster, :provided_by_gcp, :production_environment, projects: [project]) }
-
           it "destroys and redirects back to clusters list" do
             expect { go }
               .to change { Clusters::Cluster.count }.by(-1)
@@ -658,11 +566,11 @@ describe Projects::ClustersController do
       it { expect { go }.to be_denied_for(:user) }
       it { expect { go }.to be_denied_for(:external) }
     end
+  end
 
-    def go
-      delete :destroy, namespace_id: project.namespace,
-                       project_id: project,
-                       id: cluster
+  context 'no project_id param' do
+    it 'does not respond to any action without project_id param' do
+      expect { get :index }.to raise_error(ActionController::UrlGenerationError)
     end
   end
 end
