@@ -44,7 +44,38 @@ describe Projects::NotesController do
         .with(anything, anything, hash_including(last_fetched_at: last_fetched_at))
         .and_call_original
 
-      get :index, request_params
+      get :index, params: request_params
+    end
+
+    context 'when user notes_filter is present' do
+      let(:notes_json) { parsed_response[:notes] }
+      let!(:comment) { create(:note, noteable: issue, project: project) }
+      let!(:system_note) { create(:note, noteable: issue, project: project, system: true) }
+
+      it 'filters system notes by comments' do
+        user.set_notes_filter(UserPreference::NOTES_FILTERS[:only_comments], issue)
+
+        get :index, params: request_params
+
+        expect(notes_json.count).to eq(1)
+        expect(notes_json.first[:id].to_i).to eq(comment.id)
+      end
+
+      it 'returns all notes' do
+        user.set_notes_filter(UserPreference::NOTES_FILTERS[:all_notes], issue)
+
+        get :index, params: request_params
+
+        expect(notes_json.map { |note| note[:id].to_i }).to contain_exactly(comment.id, system_note.id)
+      end
+
+      it 'does not merge label event notes' do
+        user.set_notes_filter(UserPreference::NOTES_FILTERS[:only_comments], issue)
+
+        expect(ResourceEvents::MergeIntoNotesService).not_to receive(:new)
+
+        get :index, params: request_params
+      end
     end
 
     context 'for a discussion note' do
@@ -54,7 +85,7 @@ describe Projects::NotesController do
       let(:params) { request_params.merge(target_type: 'merge_request', target_id: note.noteable_id, html: true) }
 
       it 'responds with the expected attributes' do
-        get :index, params
+        get :index, params: params
 
         expect(note_json[:id]).to eq(note.id)
         expect(note_json[:discussion_html]).not_to be_nil
@@ -70,7 +101,7 @@ describe Projects::NotesController do
       let(:params) { request_params.merge(target_type: 'merge_request', target_id: note.noteable_id, html: true) }
 
       it 'responds with the expected attributes' do
-        get :index, params
+        get :index, params: params
 
         expect(note_json[:id]).to eq(note.id)
         expect(note_json[:discussion_html]).not_to be_nil
@@ -89,7 +120,7 @@ describe Projects::NotesController do
         let(:params) { request_params.merge(target_type: 'merge_request', target_id: merge_request.id, html: true) }
 
         it 'responds with the expected attributes' do
-          get :index, params
+          get :index, params: params
 
           expect(note_json[:id]).to eq(note.id)
           expect(note_json[:discussion_html]).not_to be_nil
@@ -102,7 +133,7 @@ describe Projects::NotesController do
         let(:params) { request_params.merge(target_type: 'commit', target_id: note.commit_id, html: true) }
 
         it 'responds with the expected attributes' do
-          get :index, params
+          get :index, params: params
 
           expect(note_json[:id]).to eq(note.id)
           expect(note_json[:discussion_html]).to be_nil
@@ -117,7 +148,7 @@ describe Projects::NotesController do
           end
 
           it 'renders 404' do
-            get :index, params
+            get :index, params: params
 
             expect(response).to have_gitlab_http_status(404)
           end
@@ -131,7 +162,7 @@ describe Projects::NotesController do
       let(:params) { request_params.merge(target_type: 'merge_request', target_id: note.noteable_id, html: true) }
 
       it 'responds with the expected attributes' do
-        get :index, params
+        get :index, params: params
 
         expect(note_json[:id]).to eq(note.id)
         expect(note_json[:html]).not_to be_nil
@@ -151,7 +182,7 @@ describe Projects::NotesController do
       end
 
       it 'filters notes that the user should not see' do
-        get :index, request_params
+        get :index, params: request_params
 
         expect(parsed_response[:notes].count).to eq(1)
         expect(note_json[:id]).to eq(note.id.to_s)
@@ -159,19 +190,19 @@ describe Projects::NotesController do
 
       it 'does not result in N+1 queries' do
         # Instantiate the controller variables to ensure QueryRecorder has an accurate base count
-        get :index, request_params
+        get :index, params: request_params
 
         RequestStore.clear!
 
         control_count = ActiveRecord::QueryRecorder.new do
-          get :index, request_params
+          get :index, params: request_params
         end.count
 
         RequestStore.clear!
 
         create_list(:discussion_note_on_issue, 2, :system, noteable: issue, project: issue.project, note: cross_reference)
 
-        expect { get :index, request_params }.not_to exceed_query_limit(control_count)
+        expect { get :index, params: request_params }.not_to exceed_query_limit(control_count)
       end
     end
   end
@@ -196,19 +227,19 @@ describe Projects::NotesController do
     end
 
     it "returns status 302 for html" do
-      post :create, request_params
+      post :create, params: request_params
 
       expect(response).to have_gitlab_http_status(302)
     end
 
     it "returns status 200 for json" do
-      post :create, request_params.merge(format: :json)
+      post :create, params: request_params.merge(format: :json)
 
       expect(response).to have_gitlab_http_status(200)
     end
 
     it 'returns discussion JSON when the return_discussion param is set' do
-      post :create, request_params.merge(format: :json, return_discussion: 'true')
+      post :create, params: request_params.merge(format: :json, return_discussion: 'true')
 
       expect(response).to have_gitlab_http_status(200)
       expect(json_response).to have_key 'discussion'
@@ -217,19 +248,19 @@ describe Projects::NotesController do
 
     context 'when merge_request_diff_head_sha present' do
       before do
-        service_params = {
+        service_params = ActionController::Parameters.new({
           note: 'some note',
           noteable_id: merge_request.id.to_s,
           noteable_type: 'MergeRequest',
           merge_request_diff_head_sha: 'sha',
           in_reply_to_discussion_id: nil
-        }
+        }).permit!
 
         expect(Notes::CreateService).to receive(:new).with(project, user, service_params).and_return(double(execute: true))
       end
 
       it "returns status 302 for html" do
-        post :create, request_params
+        post :create, params: request_params
 
         expect(response).to have_gitlab_http_status(302)
       end
@@ -251,7 +282,7 @@ describe Projects::NotesController do
       end
 
       def post_create(extra_params = {})
-        post :create, {
+        post :create, params: {
           note: { note: 'some other note', noteable_id: merge_request.id },
           namespace_id: project.namespace,
           project_id: project,
@@ -311,7 +342,7 @@ describe Projects::NotesController do
           namespace_id: project.namespace
         }
 
-        expect { post :create, request_params }.to change { issue.notes.count }.by(1)
+        expect { post :create, params: request_params }.to change { issue.notes.count }.by(1)
           .and change { locked_issue.notes.count }.by(0)
         expect(response).to have_gitlab_http_status(302)
       end
@@ -326,7 +357,7 @@ describe Projects::NotesController do
       context 'when a noteable is not found' do
         it 'returns 404 status' do
           request_params[:target_id] = 9999
-          post :create, request_params.merge(format: :json)
+          post :create, params: request_params.merge(format: :json)
 
           expect(response).to have_gitlab_http_status(404)
         end
@@ -334,19 +365,19 @@ describe Projects::NotesController do
 
       context 'when a user is a team member' do
         it 'returns 302 status for html' do
-          post :create, request_params
+          post :create, params: request_params
 
           expect(response).to have_gitlab_http_status(302)
         end
 
         it 'returns 200 status for json' do
-          post :create, request_params.merge(format: :json)
+          post :create, params: request_params.merge(format: :json)
 
           expect(response).to have_gitlab_http_status(200)
         end
 
         it 'creates a new note' do
-          expect { post :create, request_params }.to change { Note.count }.by(1)
+          expect { post :create, params: request_params }.to change { Note.count }.by(1)
         end
       end
 
@@ -356,41 +387,121 @@ describe Projects::NotesController do
         end
 
         it 'returns 404 status' do
-          post :create, request_params
+          post :create, params: request_params
 
           expect(response).to have_gitlab_http_status(404)
         end
 
         it 'does not create a new note' do
-          expect { post :create, request_params }.not_to change { Note.count }
+          expect { post :create, params: request_params }.not_to change { Note.count }
+        end
+      end
+    end
+
+    context 'when creating a note with quick actions' do
+      context 'with commands that return changes' do
+        let(:note_text) { "/award :thumbsup:\n/estimate 1d\n/spend 3h" }
+
+        it 'includes changes in commands_changes ' do
+          post :create, params: request_params.merge(note: { note: note_text }, format: :json)
+
+          expect(response).to have_gitlab_http_status(200)
+          expect(json_response['commands_changes']).to include('emoji_award', 'time_estimate', 'spend_time')
+          expect(json_response['commands_changes']).not_to include('target_project', 'title')
+        end
+      end
+
+      context 'with commands that do not return changes' do
+        let(:issue) { create(:issue, project: project) }
+        let(:other_project) { create(:project) }
+        let(:note_text) { "/move #{other_project.full_path}\n/title AAA" }
+
+        before do
+          other_project.add_developer(user)
+        end
+
+        it 'does not include changes in commands_changes' do
+          post :create, params: request_params.merge(note: { note: note_text }, target_type: 'issue', target_id: issue.id, format: :json)
+
+          expect(response).to have_gitlab_http_status(200)
+          expect(json_response['commands_changes']).not_to include('target_project', 'title')
         end
       end
     end
   end
 
   describe 'PUT update' do
-    context "should update the note with a valid issue" do
-      let(:request_params) do
-        {
-          namespace_id: project.namespace,
-          project_id: project,
-          id: note,
-          format: :json,
-          note: {
-            note: "New comment"
+    context "updates the note" do
+      context 'with a valid issue' do
+        let(:request_params) do
+          {
+            namespace_id: project.namespace,
+            project_id: project,
+            id: note,
+            format: :json,
+            note: {
+              note: "New comment"
+            }
           }
-        }
+        end
+
+        before do
+          sign_in(note.author)
+          project.add_developer(note.author)
+        end
+
+        it "updates the note content" do
+          expect { put :update, params: request_params }.to change { note.reload.note }
+        end
       end
 
-      before do
-        sign_in(note.author)
-        project.add_developer(note.author)
-      end
+      context "when the note is edited and a different issue is targeted" do
+        ##
+        # We are editing a note originally in a public issue of a public project,
+        # but the edit can be intercepted to change the target to a different, even confidential, issue
+        # see https://gitlab.com/gitlab-org/gitlab-ce/issues/57153
+        ##
 
-      it "updates the note" do
-        expect { put :update, request_params }.to change { note.reload.note }
+        let!(:confidential_issue) { create(:issue, :confidential, project: project) }
+        let(:new_content) { "splendiferous new content" }
+        let(:request_params) do
+          {
+            namespace_id: project.namespace,
+            project_id: project,
+            id: note,
+            format: :json,
+            note: {
+              note: new_content,
+              noteable_id: confidential_issue.id
+            }
+          }
+        end
+
+        before do
+          sign_in(note.author)
+          project.add_developer(note.author)
+
+          put :update, params: request_params
+        end
+
+        it 'returns success' do
+          expect(response.status).to eq 200
+        end
+
+        it 'edits the note content' do
+          expect(note.reload.note).to eq new_content
+        end
+
+        it 'does not create a note in the confidential issue' do
+          expect(confidential_issue.reload.notes).to be_empty
+        end
+
+        it "does not modify the note's issue" do
+          expect(note.noteable_id).to match note.reload.noteable_id
+        end
       end
     end
+
     context "doesnt update the note" do
       let(:issue)   { create(:issue, :confidential, project: project) }
       let(:note)    { create(:note, noteable: issue, project: project) }
@@ -410,7 +521,7 @@ describe Projects::NotesController do
             note: "New comment"
           }
         }
-        expect { put :update, request_params }.not_to change { note.reload.note }
+        expect { put :update, params: request_params }.not_to change { note.reload.note }
         expect(response).to have_gitlab_http_status(404)
       end
     end
@@ -433,13 +544,13 @@ describe Projects::NotesController do
       end
 
       it "returns status 200 for html" do
-        delete :destroy, request_params
+        delete :destroy, params: request_params
 
         expect(response).to have_gitlab_http_status(200)
       end
 
       it "deletes the note" do
-        expect { delete :destroy, request_params }.to change { Note.count }.from(1).to(0)
+        expect { delete :destroy, params: request_params }.to change { Note.count }.from(1).to(0)
       end
     end
 
@@ -450,7 +561,7 @@ describe Projects::NotesController do
       end
 
       it "returns status 404" do
-        delete :destroy, request_params
+        delete :destroy, params: request_params
 
         expect(response).to have_gitlab_http_status(404)
       end
@@ -465,17 +576,17 @@ describe Projects::NotesController do
 
     it "toggles the award emoji" do
       expect do
-        post(:toggle_award_emoji, request_params.merge(name: "thumbsup"))
+        post(:toggle_award_emoji, params: request_params.merge(name: "thumbsup"))
       end.to change { note.award_emoji.count }.by(1)
 
       expect(response).to have_gitlab_http_status(200)
     end
 
     it "removes the already awarded emoji" do
-      post(:toggle_award_emoji, request_params.merge(name: "thumbsup"))
+      post(:toggle_award_emoji, params: request_params.merge(name: "thumbsup"))
 
       expect do
-        post(:toggle_award_emoji, request_params.merge(name: "thumbsup"))
+        post(:toggle_award_emoji, params: request_params.merge(name: "thumbsup"))
       end.to change { AwardEmoji.count }.by(-1)
 
       expect(response).to have_gitlab_http_status(200)
@@ -494,7 +605,7 @@ describe Projects::NotesController do
 
       context "when the user is not authorized to resolve the note" do
         it "returns status 404" do
-          post :resolve, request_params
+          post :resolve, params: request_params
 
           expect(response).to have_gitlab_http_status(404)
         end
@@ -511,7 +622,7 @@ describe Projects::NotesController do
           end
 
           it "returns status 404" do
-            post :resolve, request_params
+            post :resolve, params: request_params
 
             expect(response).to have_gitlab_http_status(404)
           end
@@ -519,7 +630,7 @@ describe Projects::NotesController do
 
         context "when the note is resolvable" do
           it "resolves the note" do
-            post :resolve, request_params
+            post :resolve, params: request_params
 
             expect(note.reload.resolved?).to be true
             expect(note.reload.resolved_by).to eq(user)
@@ -528,17 +639,17 @@ describe Projects::NotesController do
           it "sends notifications if all discussions are resolved" do
             expect_any_instance_of(MergeRequests::ResolvedDiscussionNotificationService).to receive(:execute).with(merge_request)
 
-            post :resolve, request_params
+            post :resolve, params: request_params
           end
 
           it "returns the name of the resolving user" do
-            post :resolve, request_params.merge(html: true)
+            post :resolve, params: request_params.merge(html: true)
 
             expect(JSON.parse(response.body)["resolved_by"]).to eq(user.name)
           end
 
           it "returns status 200" do
-            post :resolve, request_params
+            post :resolve, params: request_params
 
             expect(response).to have_gitlab_http_status(200)
           end
@@ -555,7 +666,7 @@ describe Projects::NotesController do
 
       context "when the user is not authorized to resolve the note" do
         it "returns status 404" do
-          delete :unresolve, request_params
+          delete :unresolve, params: request_params
 
           expect(response).to have_gitlab_http_status(404)
         end
@@ -572,7 +683,7 @@ describe Projects::NotesController do
           end
 
           it "returns status 404" do
-            delete :unresolve, request_params
+            delete :unresolve, params: request_params
 
             expect(response).to have_gitlab_http_status(404)
           end
@@ -580,13 +691,13 @@ describe Projects::NotesController do
 
         context "when the note is resolvable" do
           it "unresolves the note" do
-            delete :unresolve, request_params
+            delete :unresolve, params: request_params
 
             expect(note.reload.resolved?).to be false
           end
 
           it "returns status 200" do
-            delete :unresolve, request_params
+            delete :unresolve, params: request_params
 
             expect(response).to have_gitlab_http_status(200)
           end
