@@ -12,21 +12,23 @@ describe Projects::RefsController do
   describe 'GET #logs_tree' do
     def default_get(format = :html)
       get :logs_tree,
-          namespace_id: project.namespace.to_param,
-          project_id: project,
-          id: 'master',
-          path: 'foo/bar/baz.html',
+          params: {
+            namespace_id: project.namespace.to_param,
+            project_id: project,
+            id: 'master',
+            path: 'foo/bar/baz.html'
+          },
           format: format
     end
 
     def xhr_get(format = :html)
-      xhr :get,
-          :logs_tree,
-          namespace_id: project.namespace.to_param,
-          project_id: project,
-          id: 'master',
-          path: 'foo/bar/baz.html',
-          format: format
+      get :logs_tree, params: {
+        namespace_id: project.namespace.to_param,
+        project_id: project,
+        id: 'master',
+        path: 'foo/bar/baz.html',
+        format: format
+      }, xhr: true
     end
 
     it 'never throws MissingTemplate' do
