@@ -11,7 +11,7 @@ describe Projects::Settings::IntegrationsController do
 
   describe 'GET show' do
     it 'renders show with 200 status code' do
-      get :show, namespace_id: project.namespace, project_id: project
+      get :show, params: { namespace_id: project.namespace, project_id: project }
 
       expect(response).to have_gitlab_http_status(200)
       expect(response).to render_template(:show)
