@@ -8,7 +8,13 @@ end
 shared_examples 'content not cached without revalidation and no-store' do
   it 'ensures content will not be cached without revalidation' do
     # Fixed in newer versions of ActivePack, it will only output a single `private`.
-    expect(subject['Cache-Control']).to eq('max-age=0, private, must-revalidate, private, no-store')
+    expect(subject['Cache-Control']).to eq('max-age=0, private, must-revalidate, no-store')
+  end
+end
+
+shared_examples 'content publicly cached' do
+  it 'ensures content is publicly cached' do
+    expect(subject['Cache-Control']).to eq('max-age=300, public')
   end
 end
 
@@ -23,7 +29,7 @@ describe UploadsController do
 
     context 'when a user does not have permissions to upload a file' do
       it "returns 401 when the user is not logged in" do
-        post :create, model: model, id: snippet.id, format: :json
+        post :create, params: { model: model, id: snippet.id }, format: :json
 
         expect(response).to have_gitlab_http_status(401)
       end
@@ -32,7 +38,7 @@ describe UploadsController do
         private_snippet = create(:personal_snippet, :private)
 
         sign_in(user)
-        post :create, model: model, id: private_snippet.id, format: :json
+        post :create, params: { model: model, id: private_snippet.id }, format: :json
 
         expect(response).to have_gitlab_http_status(404)
       end
@@ -44,25 +50,25 @@ describe UploadsController do
       end
 
       it "returns an error without file" do
-        post :create, model: model, id: snippet.id, format: :json
+        post :create, params: { model: model, id: snippet.id }, format: :json
 
         expect(response).to have_gitlab_http_status(422)
       end
 
       it "returns an error with invalid model" do
-        expect { post :create, model: 'invalid', id: snippet.id, format: :json }
+        expect { post :create, params: { model: 'invalid', id: snippet.id }, format: :json }
         .to raise_error(ActionController::UrlGenerationError)
       end
 
       it "returns 404 status when object not found" do
-        post :create, model: model, id: 9999, format: :json
+        post :create, params: { model: model, id: 9999 }, format: :json
 
         expect(response).to have_gitlab_http_status(404)
       end
 
       context 'with valid image' do
         before do
-          post :create, model: 'personal_snippet', id: snippet.id, file: jpg, format: :json
+          post :create, params: { model: 'personal_snippet', id: snippet.id, file: jpg }, format: :json
         end
 
         it 'returns a content with original filename, new link, and correct type.' do
@@ -82,7 +88,7 @@ describe UploadsController do
 
       context 'with valid non-image file' do
         before do
-          post :create, model: 'personal_snippet', id: snippet.id, file: txt, format: :json
+          post :create, params: { model: 'personal_snippet', id: snippet.id, file: txt }, format: :json
         end
 
         it 'returns a content with original filename, new link, and correct type.' do
@@ -102,7 +108,7 @@ describe UploadsController do
 
       context 'temporal with valid image' do
         subject do
-          post :create, model: 'personal_snippet', file: jpg, format: :json
+          post :create, params: { model: 'personal_snippet', file: jpg }, format: :json
         end
 
         it 'returns a content with original filename, new link, and correct type.' do
@@ -119,7 +125,7 @@ describe UploadsController do
 
       context 'temporal with valid non-image file' do
         subject do
-          post :create, model: 'personal_snippet', file: txt, format: :json
+          post :create, params: { model: 'personal_snippet', file: txt }, format: :json
         end
 
         it 'returns a content with original filename, new link, and correct type.' do
@@ -143,7 +149,7 @@ describe UploadsController do
       context 'for PNG files' do
         it 'returns Content-Disposition: inline' do
           note = create(:note, :with_attachment, project: project)
-          get :show, model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png'
+          get :show, params: { model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png' }
 
           expect(response['Content-Disposition']).to start_with('inline;')
         end
@@ -152,7 +158,7 @@ describe UploadsController do
       context 'for SVG files' do
         it 'returns Content-Disposition: attachment' do
           note = create(:note, :with_svg_attachment, project: project)
-          get :show, model: 'note', mounted_as: 'attachment', id: note.id, filename: 'unsanitized.svg'
+          get :show, params: { model: 'note', mounted_as: 'attachment', id: note.id, filename: 'unsanitized.svg' }
 
           expect(response['Content-Disposition']).to start_with('attachment;')
         end
@@ -171,7 +177,7 @@ describe UploadsController do
           end
 
           it "redirects to the sign in page" do
-            get :show, model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png"
+            get :show, params: { model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png" }
 
             expect(response).to redirect_to(new_user_session_path)
           end
@@ -179,14 +185,14 @@ describe UploadsController do
 
         context "when the user isn't blocked" do
           it "responds with status 200" do
-            get :show, model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png"
+            get :show, params: { model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
-          it_behaves_like 'content not cached without revalidation and no-store' do
+          it_behaves_like 'content publicly cached' do
             subject do
-              get :show, model: 'user', mounted_as: 'avatar', id: user.id, filename: 'dk.png'
+              get :show, params: { model: 'user', mounted_as: 'avatar', id: user.id, filename: 'dk.png' }
 
               response
             end
@@ -196,14 +202,14 @@ describe UploadsController do
 
       context "when not signed in" do
         it "responds with status 200" do
-          get :show, model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png"
+          get :show, params: { model: "user", mounted_as: "avatar", id: user.id, filename: "dk.png" }
 
           expect(response).to have_gitlab_http_status(200)
         end
 
-        it_behaves_like 'content not cached without revalidation' do
+        it_behaves_like 'content publicly cached' do
           subject do
-            get :show, model: 'user', mounted_as: 'avatar', id: user.id, filename: 'dk.png'
+            get :show, params: { model: 'user', mounted_as: 'avatar', id: user.id, filename: 'dk.png' }
 
             response
           end
@@ -221,14 +227,14 @@ describe UploadsController do
 
         context "when not signed in" do
           it "responds with status 200" do
-            get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+            get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation' do
             subject do
-              get :show, model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png'
+              get :show, params: { model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png' }
 
               response
             end
@@ -241,14 +247,14 @@ describe UploadsController do
           end
 
           it "responds with status 200" do
-            get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+            get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation and no-store' do
             subject do
-              get :show, model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png'
+              get :show, params: { model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png' }
 
               response
             end
@@ -263,7 +269,7 @@ describe UploadsController do
 
         context "when not signed in" do
           it "redirects to the sign in page" do
-            get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+            get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
             expect(response).to redirect_to(new_user_session_path)
           end
@@ -286,7 +292,7 @@ describe UploadsController do
               end
 
               it "redirects to the sign in page" do
-                get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+                get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
                 expect(response).to redirect_to(new_user_session_path)
               end
@@ -294,14 +300,14 @@ describe UploadsController do
 
             context "when the user isn't blocked" do
               it "responds with status 200" do
-                get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+                get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
                 expect(response).to have_gitlab_http_status(200)
               end
 
               it_behaves_like 'content not cached without revalidation and no-store' do
                 subject do
-                  get :show, model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png'
+                  get :show, params: { model: 'project', mounted_as: 'avatar', id: project.id, filename: 'dk.png' }
 
                   response
                 end
@@ -311,7 +317,7 @@ describe UploadsController do
 
           context "when the user doesn't have access to the project" do
             it "responds with status 404" do
-              get :show, model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png"
+              get :show, params: { model: "project", mounted_as: "avatar", id: project.id, filename: "dk.png" }
 
               expect(response).to have_gitlab_http_status(404)
             end
@@ -321,19 +327,19 @@ describe UploadsController do
     end
 
     context "when viewing a group avatar" do
-      let!(:group)   { create(:group, avatar: fixture_file_upload("spec/fixtures/dk.png", "image/png")) }
+      let!(:group) { create(:group, avatar: fixture_file_upload("spec/fixtures/dk.png", "image/png")) }
 
       context "when the group is public" do
         context "when not signed in" do
           it "responds with status 200" do
-            get :show, model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png"
+            get :show, params: { model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation' do
             subject do
-              get :show, model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png'
+              get :show, params: { model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png' }
 
               response
             end
@@ -346,14 +352,14 @@ describe UploadsController do
           end
 
           it "responds with status 200" do
-            get :show, model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png"
+            get :show, params: { model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation and no-store' do
             subject do
-              get :show, model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png'
+              get :show, params: { model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png' }
 
               response
             end
@@ -382,7 +388,7 @@ describe UploadsController do
               end
 
               it "redirects to the sign in page" do
-                get :show, model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png"
+                get :show, params: { model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png" }
 
                 expect(response).to redirect_to(new_user_session_path)
               end
@@ -390,14 +396,14 @@ describe UploadsController do
 
             context "when the user isn't blocked" do
               it "responds with status 200" do
-                get :show, model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png"
+                get :show, params: { model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png" }
 
                 expect(response).to have_gitlab_http_status(200)
               end
 
               it_behaves_like 'content not cached without revalidation and no-store' do
                 subject do
-                  get :show, model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png'
+                  get :show, params: { model: 'group', mounted_as: 'avatar', id: group.id, filename: 'dk.png' }
 
                   response
                 end
@@ -407,7 +413,7 @@ describe UploadsController do
 
           context "when the user doesn't have access to the project" do
             it "responds with status 404" do
-              get :show, model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png"
+              get :show, params: { model: "group", mounted_as: "avatar", id: group.id, filename: "dk.png" }
 
               expect(response).to have_gitlab_http_status(404)
             end
@@ -427,14 +433,14 @@ describe UploadsController do
 
         context "when not signed in" do
           it "responds with status 200" do
-            get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+            get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation' do
             subject do
-              get :show, model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png'
+              get :show, params: { model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png' }
 
               response
             end
@@ -447,14 +453,14 @@ describe UploadsController do
           end
 
           it "responds with status 200" do
-            get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+            get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
           it_behaves_like 'content not cached without revalidation and no-store' do
             subject do
-              get :show, model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png'
+              get :show, params: { model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png' }
 
               response
             end
@@ -469,7 +475,7 @@ describe UploadsController do
 
         context "when not signed in" do
           it "redirects to the sign in page" do
-            get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+            get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
             expect(response).to redirect_to(new_user_session_path)
           end
@@ -492,7 +498,7 @@ describe UploadsController do
               end
 
               it "redirects to the sign in page" do
-                get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+                get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
                 expect(response).to redirect_to(new_user_session_path)
               end
@@ -500,14 +506,14 @@ describe UploadsController do
 
             context "when the user isn't blocked" do
               it "responds with status 200" do
-                get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+                get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
                 expect(response).to have_gitlab_http_status(200)
               end
 
               it_behaves_like 'content not cached without revalidation and no-store' do
                 subject do
-                  get :show, model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png'
+                  get :show, params: { model: 'note', mounted_as: 'attachment', id: note.id, filename: 'dk.png' }
 
                   response
                 end
@@ -517,7 +523,7 @@ describe UploadsController do
 
           context "when the user doesn't have access to the project" do
             it "responds with status 404" do
-              get :show, model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png"
+              get :show, params: { model: "note", mounted_as: "attachment", id: note.id, filename: "dk.png" }
 
               expect(response).to have_gitlab_http_status(404)
             end
@@ -532,14 +538,14 @@ describe UploadsController do
 
         context 'when not signed in' do
           it 'responds with status 200' do
-            get :show, model: 'appearance', mounted_as: 'header_logo', id: appearance.id, filename: 'dk.png'
+            get :show, params: { model: 'appearance', mounted_as: 'header_logo', id: appearance.id, filename: 'dk.png' }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
-          it_behaves_like 'content not cached without revalidation' do
+          it_behaves_like 'content publicly cached' do
             subject do
-              get :show, model: 'appearance', mounted_as: 'header_logo', id: appearance.id, filename: 'dk.png'
+              get :show, params: { model: 'appearance', mounted_as: 'header_logo', id: appearance.id, filename: 'dk.png' }
 
               response
             end
@@ -552,14 +558,14 @@ describe UploadsController do
 
         context 'when not signed in' do
           it 'responds with status 200' do
-            get :show, model: 'appearance', mounted_as: 'logo', id: appearance.id, filename: 'dk.png'
+            get :show, params: { model: 'appearance', mounted_as: 'logo', id: appearance.id, filename: 'dk.png' }
 
             expect(response).to have_gitlab_http_status(200)
           end
 
-          it_behaves_like 'content not cached without revalidation' do
+          it_behaves_like 'content publicly cached' do
             subject do
-              get :show, model: 'appearance', mounted_as: 'logo', id: appearance.id, filename: 'dk.png'
+              get :show, params: { model: 'appearance', mounted_as: 'logo', id: appearance.id, filename: 'dk.png' }
 
               response
             end
@@ -573,7 +579,7 @@ describe UploadsController do
 
       context 'has a valid filename on the original file' do
         it 'successfully returns the file' do
-          get :show, model: 'appearance', mounted_as: 'favicon', id: appearance.id, filename: 'dk.png'
+          get :show, params: { model: 'appearance', mounted_as: 'favicon', id: appearance.id, filename: 'dk.png' }
 
           expect(response).to have_gitlab_http_status(200)
           expect(response.header['Content-Disposition']).to end_with 'filename="dk.png"'
@@ -582,7 +588,7 @@ describe UploadsController do
 
       context 'has an invalid filename on the original file' do
         it 'returns a 404' do
-          get :show, model: 'appearance', mounted_as: 'favicon', id: appearance.id, filename: 'bogus.png'
+          get :show, params: { model: 'appearance', mounted_as: 'favicon', id: appearance.id, filename: 'bogus.png' }
 
           expect(response).to have_gitlab_http_status(404)
         end
