@@ -2,13 +2,28 @@ FactoryBot.define do
   factory :cluster, class: Clusters::Cluster do
     user
     name 'test-cluster'
+    cluster_type :project_type
+
+    trait :instance do
+      cluster_type { Clusters::Cluster.cluster_types[:instance_type] }
+    end
 
     trait :project do
+      cluster_type { Clusters::Cluster.cluster_types[:project_type] }
+
       before(:create) do |cluster, evaluator|
         cluster.projects << create(:project, :repository)
       end
     end
 
+    trait :group do
+      cluster_type { Clusters::Cluster.cluster_types[:group_type] }
+
+      before(:create) do |cluster, evalutor|
+        cluster.groups << create(:group)
+      end
+    end
+
     trait :provided_by_user do
       provider_type :user
       platform_type :kubernetes
@@ -29,6 +44,10 @@ FactoryBot.define do
       provider_gcp factory: [:cluster_provider_gcp, :creating]
     end
 
+    trait :rbac_disabled do
+      platform_kubernetes factory: [:cluster_platform_kubernetes, :configured, :rbac_disabled]
+    end
+
     trait :disabled do
       enabled false
     end
@@ -40,5 +59,9 @@ FactoryBot.define do
     trait :with_installed_helm do
       application_helm factory: %i(clusters_applications_helm installed)
     end
+
+    trait :with_domain do
+      domain 'example.com'
+    end
   end
 end
