@@ -46,6 +46,17 @@ FactoryBot.define do
     )
   end
 
+  factory :jira_cloud_service, class: JiraService do
+    project
+    active true
+    properties(
+      url: 'https://mysite.atlassian.net',
+      username: 'jira_user',
+      password: 'my-secret-password',
+      project_key: 'jira-key'
+    )
+  end
+
   factory :hipchat_service do
     project
     type 'HipchatService'
