@@ -6,24 +6,6 @@ describe 'User explores projects' do
   set(:private_project) { create(:project, :private) }
   set(:public_project) { create(:project, :public) }
 
-  shared_examples_for 'shows public projects' do
-    it 'shows projects' do
-      expect(page).to have_content(public_project.title)
-      expect(page).not_to have_content(internal_project.title)
-      expect(page).not_to have_content(private_project.title)
-      expect(page).not_to have_content(archived_project.title)
-    end
-  end
-
-  shared_examples_for 'shows public and internal projects' do
-    it 'shows projects' do
-      expect(page).to have_content(public_project.title)
-      expect(page).to have_content(internal_project.title)
-      expect(page).not_to have_content(private_project.title)
-      expect(page).not_to have_content(archived_project.title)
-    end
-  end
-
   context 'when not signed in' do
     context 'when viewing public projects' do
       before do
