@@ -31,7 +31,7 @@ describe 'Projects > Snippets > User comments on a snippet', :js do
   end
 
   it 'should have zen mode' do
-    find('.js-zen-enter').click()
+    find('.js-zen-enter').click
     expect(page).to have_selector('.fullscreen')
   end
 end
