@@ -22,13 +22,18 @@ import actions, {
   expandAllFiles,
   toggleFileDiscussions,
   saveDiffDiscussion,
+  setHighlightedRow,
   toggleTreeOpen,
   scrollToFile,
   toggleShowTreeList,
+  renderFileForDiscussionId,
+  setRenderTreeList,
+  setShowWhitespace,
 } from '~/diffs/store/actions';
+import eventHub from '~/notes/event_hub';
 import * as types from '~/diffs/store/mutation_types';
-import { reduceDiscussionsToLineCodes } from '~/notes/stores/utils';
 import axios from '~/lib/utils/axios_utils';
+import mockDiffFile from 'spec/diffs/mock_data/diff_file';
 import testAction from '../../helpers/vuex_action_helper';
 
 describe('DiffsStoreActions', () => {
@@ -93,51 +98,59 @@ describe('DiffsStoreActions', () => {
     });
   });
 
+  describe('setHighlightedRow', () => {
+    it('should set lineHash and fileHash of highlightedRow', () => {
+      testAction(setHighlightedRow, 'ABC_123', {}, [
+        { type: types.SET_HIGHLIGHTED_ROW, payload: 'ABC_123' },
+      ]);
+    });
+  });
+
   describe('assignDiscussionsToDiff', () => {
     it('should merge discussions into diffs', done => {
       const state = {
         diffFiles: [
           {
-            fileHash: 'ABC',
-            parallelDiffLines: [
+            file_hash: 'ABC',
+            parallel_diff_lines: [
               {
                 left: {
-                  lineCode: 'ABC_1_1',
+                  line_code: 'ABC_1_1',
                   discussions: [],
                 },
                 right: {
-                  lineCode: 'ABC_1_1',
+                  line_code: 'ABC_1_1',
                   discussions: [],
                 },
               },
             ],
-            highlightedDiffLines: [
+            highlighted_diff_lines: [
               {
-                lineCode: 'ABC_1_1',
+                line_code: 'ABC_1_1',
                 discussions: [],
-                oldLine: 5,
-                newLine: null,
+                old_line: 5,
+                new_line: null,
               },
             ],
-            diffRefs: {
-              baseSha: 'abc',
-              headSha: 'def',
-              startSha: 'ghi',
+            diff_refs: {
+              base_sha: 'abc',
+              head_sha: 'def',
+              start_sha: 'ghi',
             },
-            newPath: 'file1',
-            oldPath: 'file2',
+            new_path: 'file1',
+            old_path: 'file2',
           },
         ],
       };
 
       const diffPosition = {
-        baseSha: 'abc',
-        headSha: 'def',
-        startSha: 'ghi',
-        newLine: null,
-        newPath: 'file1',
-        oldLine: 5,
-        oldPath: 'file2',
+        base_sha: 'abc',
+        head_sha: 'def',
+        start_sha: 'ghi',
+        new_line: null,
+        new_path: 'file1',
+        old_line: 5,
+        old_path: 'file2',
       };
 
       const singleDiscussion = {
@@ -146,13 +159,13 @@ describe('DiffsStoreActions', () => {
         diff_file: {
           file_hash: 'ABC',
         },
-        fileHash: 'ABC',
+        file_hash: 'ABC',
         resolvable: true,
         position: diffPosition,
         original_position: diffPosition,
       };
 
-      const discussions = reduceDiscussionsToLineCodes([singleDiscussion]);
+      const discussions = [singleDiscussion];
 
       testAction(
         assignDiscussionsToDiff,
@@ -162,28 +175,25 @@ describe('DiffsStoreActions', () => {
           {
             type: types.SET_LINE_DISCUSSIONS_FOR_FILE,
             payload: {
-              fileHash: 'ABC',
-              discussions: [singleDiscussion],
+              discussion: singleDiscussion,
               diffPositionByLineCode: {
                 ABC_1_1: {
-                  baseSha: 'abc',
-                  headSha: 'def',
-                  startSha: 'ghi',
-                  newLine: null,
-                  newPath: 'file1',
-                  oldLine: 5,
-                  oldPath: 'file2',
-                  lineCode: 'ABC_1_1',
-                  positionType: 'text',
+                  base_sha: 'abc',
+                  head_sha: 'def',
+                  start_sha: 'ghi',
+                  new_line: null,
+                  new_path: 'file1',
+                  old_line: 5,
+                  old_path: 'file2',
+                  line_code: 'ABC_1_1',
+                  position_type: 'text',
                 },
               },
             },
           },
         ],
         [],
-        () => {
-          done();
-        },
+        done,
       );
     });
   });
@@ -193,11 +203,11 @@ describe('DiffsStoreActions', () => {
       const state = {
         diffFiles: [
           {
-            fileHash: 'ABC',
-            parallelDiffLines: [
+            file_hash: 'ABC',
+            parallel_diff_lines: [
               {
                 left: {
-                  lineCode: 'ABC_1_1',
+                  line_code: 'ABC_1_1',
                   discussions: [
                     {
                       id: 1,
@@ -205,14 +215,14 @@ describe('DiffsStoreActions', () => {
                   ],
                 },
                 right: {
-                  lineCode: 'ABC_1_1',
+                  line_code: 'ABC_1_1',
                   discussions: [],
                 },
               },
             ],
-            highlightedDiffLines: [
+            highlighted_diff_lines: [
               {
-                lineCode: 'ABC_1_1',
+                line_code: 'ABC_1_1',
                 discussions: [],
               },
             ],
@@ -220,7 +230,8 @@ describe('DiffsStoreActions', () => {
         ],
       };
       const singleDiscussion = {
-        fileHash: 'ABC',
+        id: '1',
+        file_hash: 'ABC',
         line_code: 'ABC_1_1',
       };
 
@@ -232,15 +243,14 @@ describe('DiffsStoreActions', () => {
           {
             type: types.REMOVE_LINE_DISCUSSIONS_FOR_FILE,
             payload: {
+              id: '1',
               fileHash: 'ABC',
               lineCode: 'ABC_1_1',
             },
           },
         ],
         [],
-        () => {
-          done();
-        },
+        done,
       );
     });
   });
@@ -314,13 +324,13 @@ describe('DiffsStoreActions', () => {
 
   describe('showCommentForm', () => {
     it('should call mutation to show comment form', done => {
-      const payload = { lineCode: 'lineCode' };
+      const payload = { lineCode: 'lineCode', fileHash: 'hash' };
 
       testAction(
         showCommentForm,
         payload,
         {},
-        [{ type: types.ADD_COMMENT_FORM_LINE, payload }],
+        [{ type: types.TOGGLE_LINE_HAS_FORM, payload: { ...payload, hasForm: true } }],
         [],
         done,
       );
@@ -329,13 +339,13 @@ describe('DiffsStoreActions', () => {
 
   describe('cancelCommentForm', () => {
     it('should call mutation to cancel comment form', done => {
-      const payload = { lineCode: 'lineCode' };
+      const payload = { lineCode: 'lineCode', fileHash: 'hash' };
 
       testAction(
         cancelCommentForm,
         payload,
         {},
-        [{ type: types.REMOVE_COMMENT_FORM_LINE, payload }],
+        [{ type: types.TOGGLE_LINE_HAS_FORM, payload: { ...payload, hasForm: false } }],
         [],
         done,
       );
@@ -374,27 +384,50 @@ describe('DiffsStoreActions', () => {
 
   describe('loadCollapsedDiff', () => {
     it('should fetch data and call mutation with response and the give parameter', done => {
-      const file = { hash: 123, loadCollapsedDiffUrl: '/load/collapsed/diff/url' };
+      const file = { hash: 123, load_collapsed_diff_url: '/load/collapsed/diff/url' };
       const data = { hash: 123, parallelDiffLines: [{ lineCode: 1 }] };
       const mock = new MockAdapter(axios);
+      const commit = jasmine.createSpy('commit');
       mock.onGet(file.loadCollapsedDiffUrl).reply(200, data);
 
-      testAction(
-        loadCollapsedDiff,
-        file,
-        {},
-        [
-          {
-            type: types.ADD_COLLAPSED_DIFFS,
-            payload: { file, data },
-          },
-        ],
-        [],
-        () => {
+      loadCollapsedDiff({ commit, getters: { commitId: null } }, file)
+        .then(() => {
+          expect(commit).toHaveBeenCalledWith(types.ADD_COLLAPSED_DIFFS, { file, data });
+
           mock.restore();
           done();
-        },
-      );
+        })
+        .catch(done.fail);
+    });
+
+    it('should fetch data without commit ID', () => {
+      const file = { load_collapsed_diff_url: '/load/collapsed/diff/url' };
+      const getters = {
+        commitId: null,
+      };
+
+      spyOn(axios, 'get').and.returnValue(Promise.resolve({ data: {} }));
+
+      loadCollapsedDiff({ commit() {}, getters }, file);
+
+      expect(axios.get).toHaveBeenCalledWith(file.load_collapsed_diff_url, {
+        params: { commit_id: null },
+      });
+    });
+
+    it('should fetch data with commit ID', () => {
+      const file = { load_collapsed_diff_url: '/load/collapsed/diff/url' };
+      const getters = {
+        commitId: '123',
+      };
+
+      spyOn(axios, 'get').and.returnValue(Promise.resolve({ data: {} }));
+
+      loadCollapsedDiff({ commit() {}, getters }, file);
+
+      expect(axios.get).toHaveBeenCalledWith(file.load_collapsed_diff_url, {
+        params: { commit_id: '123' },
+      });
     });
   });
 
@@ -420,7 +453,7 @@ describe('DiffsStoreActions', () => {
       const getters = {
         getDiffFileDiscussions: jasmine.createSpy().and.returnValue([{ id: 1 }]),
         diffHasAllExpandedDiscussions: jasmine.createSpy().and.returnValue(true),
-        diffHasAllCollpasedDiscussions: jasmine.createSpy().and.returnValue(false),
+        diffHasAllCollapsedDiscussions: jasmine.createSpy().and.returnValue(false),
       };
 
       const dispatch = jasmine.createSpy('dispatch');
@@ -438,7 +471,7 @@ describe('DiffsStoreActions', () => {
       const getters = {
         getDiffFileDiscussions: jasmine.createSpy().and.returnValue([{ id: 1 }]),
         diffHasAllExpandedDiscussions: jasmine.createSpy().and.returnValue(false),
-        diffHasAllCollpasedDiscussions: jasmine.createSpy().and.returnValue(true),
+        diffHasAllCollapsedDiscussions: jasmine.createSpy().and.returnValue(true),
       };
 
       const dispatch = jasmine.createSpy();
@@ -456,7 +489,7 @@ describe('DiffsStoreActions', () => {
       const getters = {
         getDiffFileDiscussions: jasmine.createSpy().and.returnValue([{ expanded: false, id: 1 }]),
         diffHasAllExpandedDiscussions: jasmine.createSpy().and.returnValue(false),
-        diffHasAllCollpasedDiscussions: jasmine.createSpy().and.returnValue(false),
+        diffHasAllCollapsedDiscussions: jasmine.createSpy().and.returnValue(false),
       };
 
       const dispatch = jasmine.createSpy();
@@ -473,7 +506,7 @@ describe('DiffsStoreActions', () => {
 
   describe('scrollToLineIfNeededInline', () => {
     const lineMock = {
-      lineCode: 'ABC_123',
+      line_code: 'ABC_123',
     };
 
     it('should not call handleLocationHash when there is not hash', () => {
@@ -524,7 +557,7 @@ describe('DiffsStoreActions', () => {
     const lineMock = {
       left: null,
       right: {
-        lineCode: 'ABC_123',
+        line_code: 'ABC_123',
       },
     };
 
@@ -579,12 +612,18 @@ describe('DiffsStoreActions', () => {
   });
 
   describe('saveDiffDiscussion', () => {
-    beforeEach(() => {
-      spyOnDependency(actions, 'getNoteFormData').and.returnValue('testData');
-      spyOnDependency(actions, 'reduceDiscussionsToLineCodes').and.returnValue('discussions');
-    });
-
     it('dispatches actions', done => {
+      const commitId = 'something';
+      const formData = {
+        diffFile: { ...mockDiffFile },
+        noteableData: {},
+      };
+      const note = {};
+      const state = {
+        commit: {
+          id: commitId,
+        },
+      };
       const dispatch = jasmine.createSpy('dispatch').and.callFake(name => {
         switch (name) {
           case 'saveNote':
@@ -598,11 +637,19 @@ describe('DiffsStoreActions', () => {
         }
       });
 
-      saveDiffDiscussion({ dispatch }, { note: {}, formData: {} })
+      saveDiffDiscussion({ state, dispatch }, { note, formData })
         .then(() => {
-          expect(dispatch.calls.argsFor(0)).toEqual(['saveNote', 'testData', { root: true }]);
-          expect(dispatch.calls.argsFor(1)).toEqual(['updateDiscussion', 'test', { root: true }]);
-          expect(dispatch.calls.argsFor(2)).toEqual(['assignDiscussionsToDiff', 'discussions']);
+          const { calls } = dispatch;
+
+          expect(calls.count()).toBe(5);
+          expect(calls.argsFor(0)).toEqual(['saveNote', jasmine.any(Object), { root: true }]);
+
+          const postData = calls.argsFor(0)[1];
+
+          expect(postData.data.note.commit_id).toBe(commitId);
+
+          expect(calls.argsFor(1)).toEqual(['updateDiscussion', 'test', { root: true }]);
+          expect(calls.argsFor(2)).toEqual(['assignDiscussionsToDiff', ['discussion']]);
         })
         .then(done)
         .catch(done.fail);
@@ -692,4 +739,114 @@ describe('DiffsStoreActions', () => {
       expect(localStorage.setItem).toHaveBeenCalledWith('mr_tree_show', true);
     });
   });
+
+  describe('renderFileForDiscussionId', () => {
+    const rootState = {
+      notes: {
+        discussions: [
+          {
+            id: '123',
+            diff_file: {
+              file_hash: 'HASH',
+            },
+          },
+          {
+            id: '456',
+            diff_file: {
+              file_hash: 'HASH',
+            },
+          },
+        ],
+      },
+    };
+    let commit;
+    let $emit;
+    let scrollToElement;
+    const state = ({ collapsed, renderIt }) => ({
+      diffFiles: [
+        {
+          file_hash: 'HASH',
+          collapsed,
+          renderIt,
+        },
+      ],
+    });
+
+    beforeEach(() => {
+      commit = jasmine.createSpy('commit');
+      scrollToElement = spyOnDependency(actions, 'scrollToElement').and.stub();
+      $emit = spyOn(eventHub, '$emit');
+    });
+
+    it('renders and expands file for the given discussion id', () => {
+      const localState = state({ collapsed: true, renderIt: false });
+
+      renderFileForDiscussionId({ rootState, state: localState, commit }, '123');
+
+      expect(commit).toHaveBeenCalledWith('RENDER_FILE', localState.diffFiles[0]);
+      expect($emit).toHaveBeenCalledTimes(1);
+      expect(scrollToElement).toHaveBeenCalledTimes(1);
+    });
+
+    it('jumps to discussion on already rendered and expanded file', () => {
+      const localState = state({ collapsed: false, renderIt: true });
+
+      renderFileForDiscussionId({ rootState, state: localState, commit }, '123');
+
+      expect(commit).not.toHaveBeenCalled();
+      expect($emit).toHaveBeenCalledTimes(1);
+      expect(scrollToElement).not.toHaveBeenCalled();
+    });
+  });
+
+  describe('setRenderTreeList', () => {
+    it('commits SET_RENDER_TREE_LIST', done => {
+      testAction(
+        setRenderTreeList,
+        true,
+        {},
+        [{ type: types.SET_RENDER_TREE_LIST, payload: true }],
+        [],
+        done,
+      );
+    });
+
+    it('sets localStorage', () => {
+      spyOn(localStorage, 'setItem').and.stub();
+
+      setRenderTreeList({ commit() {} }, true);
+
+      expect(localStorage.setItem).toHaveBeenCalledWith('mr_diff_tree_list', true);
+    });
+  });
+
+  describe('setShowWhitespace', () => {
+    it('commits SET_SHOW_WHITESPACE', done => {
+      testAction(
+        setShowWhitespace,
+        { showWhitespace: true },
+        {},
+        [{ type: types.SET_SHOW_WHITESPACE, payload: true }],
+        [],
+        done,
+      );
+    });
+
+    it('sets localStorage', () => {
+      spyOn(localStorage, 'setItem').and.stub();
+
+      setShowWhitespace({ commit() {} }, { showWhitespace: true });
+
+      expect(localStorage.setItem).toHaveBeenCalledWith('mr_show_whitespace', true);
+    });
+
+    it('calls history pushState', () => {
+      spyOn(localStorage, 'setItem').and.stub();
+      spyOn(window.history, 'pushState').and.stub();
+
+      setShowWhitespace({ commit() {} }, { showWhitespace: true, pushState: true });
+
+      expect(window.history.pushState).toHaveBeenCalled();
+    });
+  });
 });
