@@ -18,7 +18,7 @@ const getDiffFileMock = () => Object.assign({}, diffFileMockData);
 
 describe('DiffsStoreUtils', () => {
   describe('findDiffFile', () => {
-    const files = [{ fileHash: 1, name: 'one' }];
+    const files = [{ file_hash: 1, name: 'one' }];
 
     it('should return correct file', () => {
       expect(utils.findDiffFile(files, 1).name).toEqual('one');
@@ -41,13 +41,13 @@ describe('DiffsStoreUtils', () => {
 
     describe('findIndexInInlineLines', () => {
       it('should return correct index for given line numbers', () => {
-        expectSet(utils.findIndexInInlineLines, getDiffFileMock().highlightedDiffLines);
+        expectSet(utils.findIndexInInlineLines, getDiffFileMock().highlighted_diff_lines);
       });
     });
 
     describe('findIndexInParallelLines', () => {
       it('should return correct index for given line numbers', () => {
-        expectSet(utils.findIndexInParallelLines, getDiffFileMock().parallelDiffLines, {});
+        expectSet(utils.findIndexInParallelLines, getDiffFileMock().parallel_diff_lines, {});
       });
     });
   });
@@ -56,42 +56,52 @@ describe('DiffsStoreUtils', () => {
     it('should remove match line properly by regarding the bottom parameter', () => {
       const diffFile = getDiffFileMock();
       const lineNumbers = { oldLineNumber: 3, newLineNumber: 5 };
-      const inlineIndex = utils.findIndexInInlineLines(diffFile.highlightedDiffLines, lineNumbers);
-      const parallelIndex = utils.findIndexInParallelLines(diffFile.parallelDiffLines, lineNumbers);
-      const atInlineIndex = diffFile.highlightedDiffLines[inlineIndex];
-      const atParallelIndex = diffFile.parallelDiffLines[parallelIndex];
+      const inlineIndex = utils.findIndexInInlineLines(
+        diffFile.highlighted_diff_lines,
+        lineNumbers,
+      );
+      const parallelIndex = utils.findIndexInParallelLines(
+        diffFile.parallel_diff_lines,
+        lineNumbers,
+      );
+      const atInlineIndex = diffFile.highlighted_diff_lines[inlineIndex];
+      const atParallelIndex = diffFile.parallel_diff_lines[parallelIndex];
 
       utils.removeMatchLine(diffFile, lineNumbers, false);
-      expect(diffFile.highlightedDiffLines[inlineIndex]).not.toEqual(atInlineIndex);
-      expect(diffFile.parallelDiffLines[parallelIndex]).not.toEqual(atParallelIndex);
+
+      expect(diffFile.highlighted_diff_lines[inlineIndex]).not.toEqual(atInlineIndex);
+      expect(diffFile.parallel_diff_lines[parallelIndex]).not.toEqual(atParallelIndex);
 
       utils.removeMatchLine(diffFile, lineNumbers, true);
-      expect(diffFile.highlightedDiffLines[inlineIndex + 1]).not.toEqual(atInlineIndex);
-      expect(diffFile.parallelDiffLines[parallelIndex + 1]).not.toEqual(atParallelIndex);
+
+      expect(diffFile.highlighted_diff_lines[inlineIndex + 1]).not.toEqual(atInlineIndex);
+      expect(diffFile.parallel_diff_lines[parallelIndex + 1]).not.toEqual(atParallelIndex);
     });
   });
 
   describe('addContextLines', () => {
     it('should add context lines properly with bottom parameter', () => {
       const diffFile = getDiffFileMock();
-      const inlineLines = diffFile.highlightedDiffLines;
-      const parallelLines = diffFile.parallelDiffLines;
+      const inlineLines = diffFile.highlighted_diff_lines;
+      const parallelLines = diffFile.parallel_diff_lines;
       const lineNumbers = { oldLineNumber: 3, newLineNumber: 5 };
       const contextLines = [{ lineNumber: 42 }];
       const options = { inlineLines, parallelLines, contextLines, lineNumbers, bottom: true };
-      const inlineIndex = utils.findIndexInInlineLines(diffFile.highlightedDiffLines, lineNumbers);
-      const parallelIndex = utils.findIndexInParallelLines(diffFile.parallelDiffLines, lineNumbers);
+      const inlineIndex = utils.findIndexInInlineLines(inlineLines, lineNumbers);
+      const parallelIndex = utils.findIndexInParallelLines(parallelLines, lineNumbers);
       const normalizedParallelLine = {
         left: options.contextLines[0],
         right: options.contextLines[0],
       };
 
       utils.addContextLines(options);
+
       expect(inlineLines[inlineLines.length - 1]).toEqual(contextLines[0]);
       expect(parallelLines[parallelLines.length - 1]).toEqual(normalizedParallelLine);
 
       delete options.bottom;
       utils.addContextLines(options);
+
       expect(inlineLines[inlineIndex]).toEqual(contextLines[0]);
       expect(parallelLines[parallelIndex]).toEqual(normalizedParallelLine);
     });
@@ -108,30 +118,30 @@ describe('DiffsStoreUtils', () => {
         noteableType: MERGE_REQUEST_NOTEABLE_TYPE,
         diffFile,
         noteTargetLine: {
-          lineCode: '1c497fbb3a46b78edf04cc2a2fa33f67e3ffbe2a_1_3',
-          metaData: null,
-          newLine: 3,
-          oldLine: 1,
+          line_code: '1c497fbb3a46b78edf04cc2a2fa33f67e3ffbe2a_1_3',
+          meta_data: null,
+          new_line: 3,
+          old_line: 1,
         },
         diffViewType: PARALLEL_DIFF_VIEW_TYPE,
         linePosition: LINE_POSITION_LEFT,
       };
 
       const position = JSON.stringify({
-        base_sha: diffFile.diffRefs.baseSha,
-        start_sha: diffFile.diffRefs.startSha,
-        head_sha: diffFile.diffRefs.headSha,
-        old_path: diffFile.oldPath,
-        new_path: diffFile.newPath,
+        base_sha: diffFile.diff_refs.base_sha,
+        start_sha: diffFile.diff_refs.start_sha,
+        head_sha: diffFile.diff_refs.head_sha,
+        old_path: diffFile.old_path,
+        new_path: diffFile.new_path,
         position_type: TEXT_DIFF_POSITION_TYPE,
-        old_line: options.noteTargetLine.oldLine,
-        new_line: options.noteTargetLine.newLine,
+        old_line: options.noteTargetLine.old_line,
+        new_line: options.noteTargetLine.new_line,
       });
 
       const postData = {
         view: options.diffViewType,
         line_type: options.linePosition === LINE_POSITION_RIGHT ? NEW_LINE_TYPE : OLD_LINE_TYPE,
-        merge_request_diff_head_sha: diffFile.diffRefs.headSha,
+        merge_request_diff_head_sha: diffFile.diff_refs.head_sha,
         in_reply_to_discussion_id: '',
         note_project_id: '',
         target_type: options.noteableType,
@@ -140,9 +150,9 @@ describe('DiffsStoreUtils', () => {
         note: {
           noteable_type: options.noteableType,
           noteable_id: options.noteableData.id,
-          commit_id: '',
+          commit_id: undefined,
           type: DIFF_NOTE_TYPE,
-          line_code: options.noteTargetLine.lineCode,
+          line_code: options.noteTargetLine.line_code,
           note: options.note,
           position,
         },
@@ -156,8 +166,8 @@ describe('DiffsStoreUtils', () => {
 
     it('should create legacy note form data', () => {
       const diffFile = getDiffFileMock();
-      delete diffFile.diffRefs.startSha;
-      delete diffFile.diffRefs.headSha;
+      delete diffFile.diff_refs.start_sha;
+      delete diffFile.diff_refs.head_sha;
 
       noteableDataMock.targetType = MERGE_REQUEST_NOTEABLE_TYPE;
 
@@ -167,24 +177,24 @@ describe('DiffsStoreUtils', () => {
         noteableType: MERGE_REQUEST_NOTEABLE_TYPE,
         diffFile,
         noteTargetLine: {
-          lineCode: '1c497fbb3a46b78edf04cc2a2fa33f67e3ffbe2a_1_3',
-          metaData: null,
-          newLine: 3,
-          oldLine: 1,
+          line_code: '1c497fbb3a46b78edf04cc2a2fa33f67e3ffbe2a_1_3',
+          meta_data: null,
+          new_line: 3,
+          old_line: 1,
         },
         diffViewType: PARALLEL_DIFF_VIEW_TYPE,
         linePosition: LINE_POSITION_LEFT,
       };
 
       const position = JSON.stringify({
-        base_sha: diffFile.diffRefs.baseSha,
+        base_sha: diffFile.diff_refs.base_sha,
         start_sha: undefined,
         head_sha: undefined,
-        old_path: diffFile.oldPath,
-        new_path: diffFile.newPath,
+        old_path: diffFile.old_path,
+        new_path: diffFile.new_path,
         position_type: TEXT_DIFF_POSITION_TYPE,
-        old_line: options.noteTargetLine.oldLine,
-        new_line: options.noteTargetLine.newLine,
+        old_line: options.noteTargetLine.old_line,
+        new_line: options.noteTargetLine.new_line,
       });
 
       const postData = {
@@ -199,9 +209,9 @@ describe('DiffsStoreUtils', () => {
         note: {
           noteable_type: options.noteableType,
           noteable_id: options.noteableData.id,
-          commit_id: '',
+          commit_id: undefined,
           type: LEGACY_DIFF_NOTE_TYPE,
-          line_code: options.noteTargetLine.lineCode,
+          line_code: options.noteTargetLine.line_code,
           note: options.note,
           position,
         },
@@ -221,92 +231,108 @@ describe('DiffsStoreUtils', () => {
       const lines = [{ type: null }, { type: MATCH_LINE_TYPE }];
       const linesWithReferences = utils.addLineReferences(lines, lineNumbers, true);
 
-      expect(linesWithReferences[0].oldLine).toEqual(lineNumbers.oldLineNumber + 1);
-      expect(linesWithReferences[0].newLine).toEqual(lineNumbers.newLineNumber + 1);
-      expect(linesWithReferences[1].metaData.oldPos).toEqual(4);
-      expect(linesWithReferences[1].metaData.newPos).toEqual(5);
+      expect(linesWithReferences[0].old_line).toEqual(lineNumbers.oldLineNumber + 1);
+      expect(linesWithReferences[0].new_line).toEqual(lineNumbers.newLineNumber + 1);
+      expect(linesWithReferences[1].meta_data.old_pos).toEqual(4);
+      expect(linesWithReferences[1].meta_data.new_pos).toEqual(5);
     });
 
     it('should add correct line references when bottom falsy', () => {
       const lines = [{ type: null }, { type: MATCH_LINE_TYPE }, { type: null }];
       const linesWithReferences = utils.addLineReferences(lines, lineNumbers);
 
-      expect(linesWithReferences[0].oldLine).toEqual(0);
-      expect(linesWithReferences[0].newLine).toEqual(1);
-      expect(linesWithReferences[1].metaData.oldPos).toEqual(2);
-      expect(linesWithReferences[1].metaData.newPos).toEqual(3);
+      expect(linesWithReferences[0].old_line).toEqual(0);
+      expect(linesWithReferences[0].new_line).toEqual(1);
+      expect(linesWithReferences[1].meta_data.old_pos).toEqual(2);
+      expect(linesWithReferences[1].meta_data.new_pos).toEqual(3);
     });
   });
 
   describe('trimFirstCharOfLineContent', () => {
     it('trims the line when it starts with a space', () => {
-      expect(utils.trimFirstCharOfLineContent({ richText: ' diff' })).toEqual({
-        discussions: [],
-        richText: 'diff',
+      expect(utils.trimFirstCharOfLineContent({ rich_text: ' diff' })).toEqual({
+        rich_text: 'diff',
       });
     });
 
     it('trims the line when it starts with a +', () => {
-      expect(utils.trimFirstCharOfLineContent({ richText: '+diff' })).toEqual({
-        discussions: [],
-        richText: 'diff',
+      expect(utils.trimFirstCharOfLineContent({ rich_text: '+diff' })).toEqual({
+        rich_text: 'diff',
       });
     });
 
     it('trims the line when it starts with a -', () => {
-      expect(utils.trimFirstCharOfLineContent({ richText: '-diff' })).toEqual({
-        discussions: [],
-        richText: 'diff',
+      expect(utils.trimFirstCharOfLineContent({ rich_text: '-diff' })).toEqual({
+        rich_text: 'diff',
       });
     });
 
     it('does not trims the line when it starts with a letter', () => {
-      expect(utils.trimFirstCharOfLineContent({ richText: 'diff' })).toEqual({
-        discussions: [],
-        richText: 'diff',
+      expect(utils.trimFirstCharOfLineContent({ rich_text: 'diff' })).toEqual({
+        rich_text: 'diff',
       });
     });
 
     it('does not modify the provided object', () => {
       const lineObj = {
-        discussions: [],
-        richText: ' diff',
+        rich_text: ' diff',
       };
 
       utils.trimFirstCharOfLineContent(lineObj);
-      expect(lineObj).toEqual({ discussions: [], richText: ' diff' });
+
+      expect(lineObj).toEqual({ rich_text: ' diff' });
     });
 
     it('handles a undefined or null parameter', () => {
-      expect(utils.trimFirstCharOfLineContent()).toEqual({ discussions: [] });
+      expect(utils.trimFirstCharOfLineContent()).toEqual({});
     });
   });
 
   describe('prepareDiffData', () => {
+    let preparedDiff;
+
+    beforeEach(() => {
+      preparedDiff = { diff_files: [getDiffFileMock()] };
+      utils.prepareDiffData(preparedDiff);
+    });
+
     it('sets the renderIt and collapsed attribute on files', () => {
-      const preparedDiff = { diffFiles: [getDiffFileMock()] };
-      utils.prepareDiffData(preparedDiff);
+      const firstParallelDiffLine = preparedDiff.diff_files[0].parallel_diff_lines[2];
 
-      const firstParallelDiffLine = preparedDiff.diffFiles[0].parallelDiffLines[2];
       expect(firstParallelDiffLine.left.discussions.length).toBe(0);
       expect(firstParallelDiffLine.left).not.toHaveAttr('text');
       expect(firstParallelDiffLine.right.discussions.length).toBe(0);
       expect(firstParallelDiffLine.right).not.toHaveAttr('text');
-      const firstParallelChar = firstParallelDiffLine.right.richText.charAt(0);
+      const firstParallelChar = firstParallelDiffLine.right.rich_text.charAt(0);
+
       expect(firstParallelChar).not.toBe(' ');
       expect(firstParallelChar).not.toBe('+');
       expect(firstParallelChar).not.toBe('-');
 
-      const checkLine = preparedDiff.diffFiles[0].highlightedDiffLines[0];
+      const checkLine = preparedDiff.diff_files[0].highlighted_diff_lines[0];
+
       expect(checkLine.discussions.length).toBe(0);
       expect(checkLine).not.toHaveAttr('text');
-      const firstChar = checkLine.richText.charAt(0);
+      const firstChar = checkLine.rich_text.charAt(0);
+
       expect(firstChar).not.toBe(' ');
       expect(firstChar).not.toBe('+');
       expect(firstChar).not.toBe('-');
 
-      expect(preparedDiff.diffFiles[0].renderIt).toBeTruthy();
-      expect(preparedDiff.diffFiles[0].collapsed).toBeFalsy();
+      expect(preparedDiff.diff_files[0].renderIt).toBeTruthy();
+      expect(preparedDiff.diff_files[0].collapsed).toBeFalsy();
+    });
+
+    it('adds line_code to all lines', () => {
+      expect(
+        preparedDiff.diff_files[0].parallel_diff_lines.filter(line => !line.line_code),
+      ).toHaveLength(0);
+    });
+
+    it('uses right line code if left has none', () => {
+      const firstLine = preparedDiff.diff_files[0].parallel_diff_lines[0];
+
+      expect(firstLine.line_code).toEqual(firstLine.right.line_code);
     });
   });
 
@@ -389,7 +415,7 @@ describe('DiffsStoreUtils', () => {
           discussion,
           diffPosition: {
             ...diffPosition,
-            lineCode: 'ABC_1',
+            line_code: 'ABC_1',
           },
           latestDiff: true,
         }),
@@ -420,28 +446,36 @@ describe('DiffsStoreUtils', () => {
     beforeAll(() => {
       files = [
         {
-          newPath: 'app/index.js',
-          deletedFile: false,
-          newFile: false,
-          removedLines: 10,
-          addedLines: 0,
-          fileHash: 'test',
+          new_path: 'app/index.js',
+          deleted_file: false,
+          new_file: false,
+          removed_lines: 10,
+          added_lines: 0,
+          file_hash: 'test',
         },
         {
-          newPath: 'app/test/index.js',
-          deletedFile: false,
-          newFile: true,
-          removedLines: 0,
-          addedLines: 0,
-          fileHash: 'test',
+          new_path: 'app/test/index.js',
+          deleted_file: false,
+          new_file: true,
+          removed_lines: 0,
+          added_lines: 0,
+          file_hash: 'test',
         },
         {
-          newPath: 'package.json',
-          deletedFile: true,
-          newFile: false,
-          removedLines: 0,
-          addedLines: 0,
-          fileHash: 'test',
+          new_path: 'app/test/filepathneedstruncating.js',
+          deleted_file: false,
+          new_file: true,
+          removed_lines: 0,
+          added_lines: 0,
+          file_hash: 'test',
+        },
+        {
+          new_path: 'package.json',
+          deleted_file: true,
+          new_file: false,
+          removed_lines: 0,
+          added_lines: 0,
+          file_hash: 'test',
         },
       ];
     });
@@ -463,6 +497,7 @@ describe('DiffsStoreUtils', () => {
               fileHash: 'test',
               key: 'app/index.js',
               name: 'index.js',
+              parentPath: 'app/',
               path: 'app/index.js',
               removedLines: 10,
               tempFile: false,
@@ -483,12 +518,27 @@ describe('DiffsStoreUtils', () => {
                   fileHash: 'test',
                   key: 'app/test/index.js',
                   name: 'index.js',
+                  parentPath: 'app/test/',
                   path: 'app/test/index.js',
                   removedLines: 0,
                   tempFile: true,
                   type: 'blob',
                   tree: [],
                 },
+                {
+                  addedLines: 0,
+                  changed: true,
+                  deleted: false,
+                  fileHash: 'test',
+                  key: 'app/test/filepathneedstruncating.js',
+                  name: 'filepathneedstruncating.js',
+                  parentPath: 'app/test/',
+                  path: 'app/test/filepathneedstruncating.js',
+                  removedLines: 0,
+                  tempFile: true,
+                  type: 'blob',
+                  tree: [],
+                },
               ],
             },
           ],
@@ -496,6 +546,7 @@ describe('DiffsStoreUtils', () => {
         },
         {
           key: 'package.json',
+          parentPath: '/',
           path: 'package.json',
           name: 'package.json',
           type: 'blob',
@@ -518,8 +569,202 @@ describe('DiffsStoreUtils', () => {
         'app/index.js',
         'app/test',
         'app/test/index.js',
+        'app/test/filepathneedstruncating.js',
         'package.json',
       ]);
     });
   });
+
+  describe('getDiffMode', () => {
+    it('returns mode when matched in file', () => {
+      expect(
+        utils.getDiffMode({
+          renamed_file: true,
+        }),
+      ).toBe('renamed');
+    });
+
+    it('returns mode_changed if key has no match', () => {
+      expect(
+        utils.getDiffMode({
+          mode_changed: true,
+        }),
+      ).toBe('mode_changed');
+    });
+
+    it('defaults to replaced', () => {
+      expect(utils.getDiffMode({})).toBe('replaced');
+    });
+  });
+
+  describe('getLowestSingleFolder', () => {
+    it('returns path and tree of lowest single folder tree', () => {
+      const folder = {
+        name: 'app',
+        type: 'tree',
+        tree: [
+          {
+            name: 'javascripts',
+            type: 'tree',
+            tree: [
+              {
+                type: 'blob',
+                name: 'index.js',
+              },
+            ],
+          },
+        ],
+      };
+      const { path, treeAcc } = utils.getLowestSingleFolder(folder);
+
+      expect(path).toEqual('app/javascripts');
+      expect(treeAcc).toEqual([
+        {
+          type: 'blob',
+          name: 'index.js',
+        },
+      ]);
+    });
+
+    it('returns passed in folders path & tree when more than tree exists', () => {
+      const folder = {
+        name: 'app',
+        type: 'tree',
+        tree: [
+          {
+            name: 'spec',
+            type: 'blob',
+            tree: [],
+          },
+        ],
+      };
+      const { path, treeAcc } = utils.getLowestSingleFolder(folder);
+
+      expect(path).toEqual('app');
+      expect(treeAcc).toBeNull();
+    });
+  });
+
+  describe('flattenTree', () => {
+    it('returns flattened directory structure', () => {
+      const tree = [
+        {
+          type: 'tree',
+          name: 'app',
+          tree: [
+            {
+              type: 'tree',
+              name: 'javascripts',
+              tree: [
+                {
+                  type: 'blob',
+                  name: 'index.js',
+                  tree: [],
+                },
+              ],
+            },
+          ],
+        },
+        {
+          type: 'tree',
+          name: 'ee',
+          tree: [
+            {
+              type: 'tree',
+              name: 'lib',
+              tree: [
+                {
+                  type: 'tree',
+                  name: 'ee',
+                  tree: [
+                    {
+                      type: 'tree',
+                      name: 'gitlab',
+                      tree: [
+                        {
+                          type: 'tree',
+                          name: 'checks',
+                          tree: [
+                            {
+                              type: 'tree',
+                              name: 'longtreenametomakepath',
+                              tree: [
+                                {
+                                  type: 'blob',
+                                  name: 'diff_check.rb',
+                                  tree: [],
+                                },
+                              ],
+                            },
+                          ],
+                        },
+                      ],
+                    },
+                  ],
+                },
+              ],
+            },
+          ],
+        },
+        {
+          type: 'tree',
+          name: 'spec',
+          tree: [
+            {
+              type: 'tree',
+              name: 'javascripts',
+              tree: [],
+            },
+            {
+              type: 'blob',
+              name: 'index_spec.js',
+              tree: [],
+            },
+          ],
+        },
+      ];
+      const flattened = utils.flattenTree(tree);
+
+      expect(flattened).toEqual([
+        {
+          type: 'tree',
+          name: 'app/javascripts',
+          tree: [
+            {
+              type: 'blob',
+              name: 'index.js',
+              tree: [],
+            },
+          ],
+        },
+        {
+          type: 'tree',
+          name: 'ee/lib/…/…/…/longtreenametomakepath',
+          tree: [
+            {
+              name: 'diff_check.rb',
+              tree: [],
+              type: 'blob',
+            },
+          ],
+        },
+        {
+          type: 'tree',
+          name: 'spec',
+          tree: [
+            {
+              type: 'tree',
+              name: 'javascripts',
+              tree: [],
+            },
+            {
+              type: 'blob',
+              name: 'index_spec.js',
+              tree: [],
+            },
+          ],
+        },
+      ]);
+    });
+  });
 });
