@@ -22,9 +22,10 @@ describe Projects::BranchesController, '(JavaScript fixtures)', type: :controlle
   end
 
   it 'branches/new_branch.html.raw' do |example|
-    get :new,
+    get :new, params: {
       namespace_id: project.namespace.to_param,
       project_id: project
+    }
 
     expect(response).to be_success
     store_frontend_fixture(response, example.description)
