@@ -165,7 +165,6 @@ export const note = {
   report_abuse_path:
     '/abuse_reports/new?ref_url=http%3A%2F%2Flocalhost%3A3000%2Fgitlab-org%2Fgitlab-ce%2Fissues%2F7%23note_546&user_id=1',
   path: '/gitlab-org/gitlab-ce/notes/546',
-  cached_markdown_version: 11,
 };
 
 export const discussionMock = {
@@ -305,6 +304,7 @@ export const discussionMock = {
   ],
   individual_note: false,
   resolvable: true,
+  active: true,
 };
 
 export const loggedOutnoteableData = {
@@ -1173,6 +1173,7 @@ export const discussion1 = {
   id: 'abc1',
   resolvable: true,
   resolved: false,
+  active: true,
   diff_file: {
     file_path: 'about.md',
   },
@@ -1209,6 +1210,7 @@ export const discussion2 = {
   id: 'abc2',
   resolvable: true,
   resolved: false,
+  active: true,
   diff_file: {
     file_path: 'README.md',
   },
@@ -1226,6 +1228,7 @@ export const discussion2 = {
 export const discussion3 = {
   id: 'abc3',
   resolvable: true,
+  active: true,
   resolved: false,
   diff_file: {
     file_path: 'README.md',
@@ -1244,3 +1247,18 @@ export const discussion3 = {
 export const unresolvableDiscussion = {
   resolvable: false,
 };
+
+export const discussionFiltersMock = [
+  {
+    title: 'Show all activity',
+    value: 0,
+  },
+  {
+    title: 'Show comments only',
+    value: 1,
+  },
+  {
+    title: 'Show system notes only',
+    value: 2,
+  },
+];
