@@ -12,7 +12,7 @@ describe('Page component', () => {
   let testPage;
 
   beforeEach(done => {
-    pdfjsLib.PDFJS.workerSrc = workerSrc;
+    pdfjsLib.GlobalWorkerOptions.workerSrc = workerSrc;
     pdfjsLib
       .getDocument(testPDF)
       .then(pdf => pdf.getPage(1))
@@ -34,6 +34,7 @@ describe('Page component', () => {
       page: testPage,
       number: 1,
     });
+
     expect(vm.rendering).toBe(true);
 
     promise
