@@ -15,8 +15,13 @@ describe('NothingToMerge', () => {
     it('should have correct elements', () => {
       expect(vm.$el.classList.contains('mr-widget-body')).toBeTruthy();
       expect(vm.$el.querySelector('a').href).toContain(newBlobPath);
-      expect(vm.$el.innerText).toContain('Currently there are no changes in this merge request\'s source branch');
-      expect(vm.$el.innerText).toContain('Please push new commits or use a different branch.');
+      expect(vm.$el.innerText).toContain(
+        "Currently there are no changes in this merge request's source branch",
+      );
+
+      expect(vm.$el.innerText.replace(/\s\s+/g, ' ')).toContain(
+        'Please push new commits or use a different branch.',
+      );
     });
 
     it('should not show new blob link if there is no link available', () => {
