@@ -5,18 +5,21 @@ import notify from '~/lib/utils/notify';
 import { stateKey } from '~/vue_merge_request_widget/stores/state_maps';
 import mountComponent from 'spec/helpers/vue_mount_component_helper';
 import mockData from './mock_data';
-import { faviconDataUrl, overlayDataUrl, faviconWithOverlayDataUrl } from '../lib/utils/mock_data';
+import { faviconDataUrl, overlayDataUrl } from '../lib/utils/mock_data';
 
-const returnPromise = data => new Promise((resolve) => {
-  resolve({
-    data,
+const returnPromise = data =>
+  new Promise(resolve => {
+    resolve({
+      data,
+    });
   });
-});
 
 describe('mrWidgetOptions', () => {
   let vm;
   let MrWidgetOptions;
 
+  const COLLABORATION_MESSAGE = 'Allows commits from members who can merge to the target branch';
+
   beforeEach(() => {
     // Prevent component mounting
     delete mrWidgetOptions.el;
@@ -46,6 +49,7 @@ describe('mrWidgetOptions', () => {
 
       it('should return conflicts component', () => {
         vm.mr.state = 'conflicts';
+
         expect(vm.componentName).toEqual('mr-widget-conflicts');
       });
     });
@@ -57,6 +61,7 @@ describe('mrWidgetOptions', () => {
 
       it('should return true for a state which requires help widget', () => {
         vm.mr.state = 'conflicts';
+
         expect(vm.shouldRenderMergeHelp).toBeTruthy();
       });
     });
@@ -82,6 +87,7 @@ describe('mrWidgetOptions', () => {
 
       it('should return true if there is relatedLinks in MR', () => {
         Vue.set(vm.mr, 'relatedLinks', {});
+
         expect(vm.shouldRenderRelatedLinks).toBeTruthy();
       });
     });
@@ -128,11 +134,58 @@ describe('mrWidgetOptions', () => {
         expect(vm.shouldRenderSourceBranchRemovalStatus).toEqual(false);
       });
     });
+
+    describe('shouldRenderCollaborationStatus', () => {
+      describe('when collaboration is allowed', () => {
+        beforeEach(() => {
+          vm.mr.allowCollaboration = true;
+        });
+
+        describe('when merge request is opened', () => {
+          beforeEach(done => {
+            vm.mr.isOpen = true;
+            vm.$nextTick(done);
+          });
+
+          it('should render collaboration status', () => {
+            expect(vm.$el.textContent).toContain(COLLABORATION_MESSAGE);
+          });
+        });
+
+        describe('when merge request is not opened', () => {
+          beforeEach(done => {
+            vm.mr.isOpen = false;
+            vm.$nextTick(done);
+          });
+
+          it('should not render collaboration status', () => {
+            expect(vm.$el.textContent).not.toContain(COLLABORATION_MESSAGE);
+          });
+        });
+      });
+
+      describe('when collaboration is not allowed', () => {
+        beforeEach(() => {
+          vm.mr.allowCollaboration = false;
+        });
+
+        describe('when merge request is opened', () => {
+          beforeEach(done => {
+            vm.mr.isOpen = true;
+            vm.$nextTick(done);
+          });
+
+          it('should not render collaboration status', () => {
+            expect(vm.$el.textContent).not.toContain(COLLABORATION_MESSAGE);
+          });
+        });
+      });
+    });
   });
 
   describe('methods', () => {
     describe('checkStatus', () => {
-      it('should tell service to check status', (done) => {
+      it('should tell service to check status', done => {
         spyOn(vm.service, 'checkStatus').and.returnValue(returnPromise(mockData));
         spyOn(vm.mr, 'setData');
         spyOn(vm, 'handleNotification');
@@ -182,10 +235,10 @@ describe('mrWidgetOptions', () => {
     });
 
     describe('fetchDeployments', () => {
-      it('should fetch deployments', (done) => {
+      it('should fetch deployments', done => {
         spyOn(vm.service, 'fetchDeployments').and.returnValue(returnPromise([{ id: 1 }]));
 
-        vm.fetchDeployments();
+        vm.fetchPreMergeDeployments();
 
         setTimeout(() => {
           expect(vm.service.fetchDeployments).toHaveBeenCalled();
@@ -197,7 +250,7 @@ describe('mrWidgetOptions', () => {
     });
 
     describe('fetchActionsContent', () => {
-      it('should fetch content of Cherry Pick and Revert modals', (done) => {
+      it('should fetch content of Cherry Pick and Revert modals', done => {
         spyOn(vm.service, 'fetchMergeActionsContent').and.returnValue(returnPromise('hello world'));
 
         vm.fetchActionsContent();
@@ -223,18 +276,23 @@ describe('mrWidgetOptions', () => {
         vm.bindEventHubListeners();
 
         eventHub.$emit('SetBranchRemoveFlag', ['flag']);
+
         expect(vm.mr.isRemovingSourceBranch).toEqual('flag');
 
         eventHub.$emit('FailedToMerge');
+
         expect(vm.mr.state).toEqual('failedToMerge');
 
         eventHub.$emit('UpdateWidgetData', mockData);
+
         expect(vm.mr.setData).toHaveBeenCalledWith(mockData);
 
         eventHub.$emit('EnablePolling');
+
         expect(vm.resumePolling).toHaveBeenCalled();
 
         eventHub.$emit('DisablePolling');
+
         expect(vm.stopPolling).toHaveBeenCalled();
 
         const listenersWithServiceRequest = {
@@ -243,7 +301,7 @@ describe('mrWidgetOptions', () => {
         };
 
         const allArgs = eventHub.$on.calls.allArgs();
-        allArgs.forEach((params) => {
+        allArgs.forEach(params => {
           const eventName = params[0];
           const callback = params[1];
 
@@ -253,25 +311,15 @@ describe('mrWidgetOptions', () => {
         });
 
         listenersWithServiceRequest.MRWidgetUpdateRequested();
+
         expect(vm.checkStatus).toHaveBeenCalled();
 
         listenersWithServiceRequest.FetchActionsContent();
+
         expect(vm.fetchActionsContent).toHaveBeenCalled();
       });
     });
 
-    describe('handleMounted', () => {
-      it('should call required methods to do the initial kick-off', () => {
-        spyOn(vm, 'initDeploymentsPolling');
-        spyOn(vm, 'setFaviconHelper');
-
-        vm.handleMounted();
-
-        expect(vm.setFaviconHelper).toHaveBeenCalled();
-        expect(vm.initDeploymentsPolling).toHaveBeenCalled();
-      });
-    });
-
     describe('setFavicon', () => {
       let faviconElement;
 
@@ -288,20 +336,31 @@ describe('mrWidgetOptions', () => {
         document.body.removeChild(document.getElementById('favicon'));
       });
 
-      it('should call setFavicon method', (done) => {
+      it('should call setFavicon method', done => {
         vm.mr.ciStatusFaviconPath = overlayDataUrl;
-        vm.setFaviconHelper().then(() => {
-          expect(faviconElement.getAttribute('href')).toEqual(faviconWithOverlayDataUrl);
-          done();
-        })
-        .catch(done.fail);
+        vm.setFaviconHelper()
+          .then(() => {
+            /*
+            It would be better if we'd could mock commonUtils.setFaviconURL
+            with a spy and test that it was called. We are doing the following
+            tests as a proxy to show that the function has been called
+            */
+            expect(faviconElement.getAttribute('href')).not.toEqual(null);
+            expect(faviconElement.getAttribute('href')).not.toEqual(overlayDataUrl);
+            expect(faviconElement.getAttribute('href')).not.toEqual(faviconDataUrl);
+            done();
+          })
+          .catch(done.fail);
       });
 
-      it('should not call setFavicon when there is no ciStatusFaviconPath', () => {
+      it('should not call setFavicon when there is no ciStatusFaviconPath', done => {
         vm.mr.ciStatusFaviconPath = null;
-        vm.setFaviconHelper();
-
-        expect(faviconElement.getAttribute('href')).toEqual(null);
+        vm.setFaviconHelper()
+          .then(() => {
+            expect(faviconElement.getAttribute('href')).toEqual(null);
+            done();
+          })
+          .catch(done.fail);
       });
     });
 
@@ -352,6 +411,7 @@ describe('mrWidgetOptions', () => {
         spyOn(vm.pollingInterval, 'resume');
 
         vm.resumePolling();
+
         expect(vm.pollingInterval.resume).toHaveBeenCalled();
       });
     });
@@ -361,13 +421,14 @@ describe('mrWidgetOptions', () => {
         spyOn(vm.pollingInterval, 'stopTimer');
 
         vm.stopPolling();
+
         expect(vm.pollingInterval.stopTimer).toHaveBeenCalled();
       });
     });
   });
 
   describe('rendering relatedLinks', () => {
-    beforeEach((done) => {
+    beforeEach(done => {
       vm.mr.relatedLinks = {
         assignToMe: null,
         closing: `
@@ -384,7 +445,7 @@ describe('mrWidgetOptions', () => {
       expect(vm.$el.querySelector('.close-related-link')).toBeDefined();
     });
 
-    it('does not render if state is nothingToMerge', (done) => {
+    it('does not render if state is nothingToMerge', done => {
       vm.mr.state = stateKey.nothingToMerge;
       Vue.nextTick(() => {
         expect(vm.$el.querySelector('.close-related-link')).toBeNull();
@@ -394,7 +455,7 @@ describe('mrWidgetOptions', () => {
   });
 
   describe('rendering source branch removal status', () => {
-    it('renders when user cannot remove branch and branch should be removed', (done) => {
+    it('renders when user cannot remove branch and branch should be removed', done => {
       vm.mr.canRemoveSourceBranch = false;
       vm.mr.shouldRemoveSourceBranch = true;
       vm.mr.state = 'readyToMerge';
@@ -402,7 +463,7 @@ describe('mrWidgetOptions', () => {
       vm.$nextTick(() => {
         const tooltip = vm.$el.querySelector('.fa-question-circle');
 
-        expect(vm.$el.textContent).toContain('Removes source branch');
+        expect(vm.$el.textContent).toContain('Deletes source branch');
         expect(tooltip.getAttribute('data-original-title')).toBe(
           'A user with write access to the source branch selected this option',
         );
@@ -411,14 +472,14 @@ describe('mrWidgetOptions', () => {
       });
     });
 
-    it('does not render in merged state', (done) => {
+    it('does not render in merged state', done => {
       vm.mr.canRemoveSourceBranch = false;
       vm.mr.shouldRemoveSourceBranch = true;
       vm.mr.state = 'merged';
 
       vm.$nextTick(() => {
-        expect(vm.$el.textContent).toContain('The source branch has been removed');
-        expect(vm.$el.textContent).not.toContain('Removes source branch');
+        expect(vm.$el.textContent).toContain('The source branch has been deleted');
+        expect(vm.$el.textContent).not.toContain('Deletes source branch');
 
         done();
       });
@@ -426,6 +487,20 @@ describe('mrWidgetOptions', () => {
   });
 
   describe('rendering deployments', () => {
+    const changes = [
+      {
+        path: 'index.html',
+        external_url: 'http://root-master-patch-91341.volatile-watch.surge.sh/index.html',
+      },
+      {
+        path: 'imgs/gallery.html',
+        external_url: 'http://root-master-patch-91341.volatile-watch.surge.sh/imgs/gallery.html',
+      },
+      {
+        path: 'about/',
+        external_url: 'http://root-master-patch-91341.volatile-watch.surge.sh/about/',
+      },
+    ];
     const deploymentMockData = {
       id: 15,
       name: 'review/diplo',
@@ -437,15 +512,20 @@ describe('mrWidgetOptions', () => {
       external_url_formatted: 'diplo.',
       deployed_at: '2017-03-22T22:44:42.258Z',
       deployed_at_formatted: 'Mar 22, 2017 10:44pm',
+      changes,
+      status: 'success',
     };
 
-    beforeEach((done) => {
-      vm.mr.deployments.push({
-        ...deploymentMockData,
-      }, {
-        ...deploymentMockData,
-        id: deploymentMockData.id + 1,
-      });
+    beforeEach(done => {
+      vm.mr.deployments.push(
+        {
+          ...deploymentMockData,
+        },
+        {
+          ...deploymentMockData,
+          id: deploymentMockData.id + 1,
+        },
+      );
 
       vm.$nextTick(done);
     });
@@ -453,5 +533,201 @@ describe('mrWidgetOptions', () => {
     it('renders multiple deployments', () => {
       expect(vm.$el.querySelectorAll('.deploy-heading').length).toBe(2);
     });
+
+    it('renders dropdpown with multiple file changes', () => {
+      expect(
+        vm.$el
+          .querySelector('.js-mr-wigdet-deployment-dropdown')
+          .querySelectorAll('.js-filtered-dropdown-result').length,
+      ).toEqual(changes.length);
+    });
+  });
+
+  describe('pipeline for target branch after merge', () => {
+    describe('with information for target branch pipeline', () => {
+      beforeEach(done => {
+        vm.mr.state = 'merged';
+        vm.mr.mergePipeline = {
+          id: 127,
+          user: {
+            id: 1,
+            name: 'Administrator',
+            username: 'root',
+            state: 'active',
+            avatar_url: null,
+            web_url: 'http://localhost:3000/root',
+            status_tooltip_html: null,
+            path: '/root',
+          },
+          active: true,
+          coverage: null,
+          source: 'push',
+          created_at: '2018-10-22T11:41:35.186Z',
+          updated_at: '2018-10-22T11:41:35.433Z',
+          path: '/root/ci-web-terminal/pipelines/127',
+          flags: {
+            latest: true,
+            stuck: true,
+            auto_devops: false,
+            yaml_errors: false,
+            retryable: false,
+            cancelable: true,
+            failure_reason: false,
+          },
+          details: {
+            status: {
+              icon: 'status_pending',
+              text: 'pending',
+              label: 'pending',
+              group: 'pending',
+              tooltip: 'pending',
+              has_details: true,
+              details_path: '/root/ci-web-terminal/pipelines/127',
+              illustration: null,
+              favicon:
+                '/assets/ci_favicons/favicon_status_pending-5bdf338420e5221ca24353b6bff1c9367189588750632e9a871b7af09ff6a2ae.png',
+            },
+            duration: null,
+            finished_at: null,
+            stages: [
+              {
+                name: 'test',
+                title: 'test: pending',
+                status: {
+                  icon: 'status_pending',
+                  text: 'pending',
+                  label: 'pending',
+                  group: 'pending',
+                  tooltip: 'pending',
+                  has_details: true,
+                  details_path: '/root/ci-web-terminal/pipelines/127#test',
+                  illustration: null,
+                  favicon:
+                    '/assets/ci_favicons/favicon_status_pending-5bdf338420e5221ca24353b6bff1c9367189588750632e9a871b7af09ff6a2ae.png',
+                },
+                path: '/root/ci-web-terminal/pipelines/127#test',
+                dropdown_path: '/root/ci-web-terminal/pipelines/127/stage.json?stage=test',
+              },
+            ],
+            artifacts: [],
+            manual_actions: [],
+            scheduled_actions: [],
+          },
+          ref: {
+            name: 'master',
+            path: '/root/ci-web-terminal/commits/master',
+            tag: false,
+            branch: true,
+          },
+          commit: {
+            id: 'aa1939133d373c94879becb79d91828a892ee319',
+            short_id: 'aa193913',
+            title: "Merge branch 'master-test' into 'master'",
+            created_at: '2018-10-22T11:41:33.000Z',
+            parent_ids: [
+              '4622f4dd792468993003caf2e3be978798cbe096',
+              '76598df914cdfe87132d0c3c40f80db9fa9396a4',
+            ],
+            message:
+              "Merge branch 'master-test' into 'master'\n\nUpdate .gitlab-ci.yml\n\nSee merge request root/ci-web-terminal!1",
+            author_name: 'Administrator',
+            author_email: 'admin@example.com',
+            authored_date: '2018-10-22T11:41:33.000Z',
+            committer_name: 'Administrator',
+            committer_email: 'admin@example.com',
+            committed_date: '2018-10-22T11:41:33.000Z',
+            author: {
+              id: 1,
+              name: 'Administrator',
+              username: 'root',
+              state: 'active',
+              avatar_url: null,
+              web_url: 'http://localhost:3000/root',
+              status_tooltip_html: null,
+              path: '/root',
+            },
+            author_gravatar_url: null,
+            commit_url:
+              'http://localhost:3000/root/ci-web-terminal/commit/aa1939133d373c94879becb79d91828a892ee319',
+            commit_path: '/root/ci-web-terminal/commit/aa1939133d373c94879becb79d91828a892ee319',
+          },
+          cancel_path: '/root/ci-web-terminal/pipelines/127/cancel',
+        };
+        vm.$nextTick(done);
+      });
+
+      it('renders pipeline block', () => {
+        expect(vm.$el.querySelector('.js-post-merge-pipeline')).not.toBeNull();
+      });
+
+      describe('with post merge deployments', () => {
+        beforeEach(done => {
+          vm.mr.postMergeDeployments = [
+            {
+              id: 15,
+              name: 'review/diplo',
+              url: '/root/acets-review-apps/environments/15',
+              stop_url: '/root/acets-review-apps/environments/15/stop',
+              metrics_url: '/root/acets-review-apps/environments/15/deployments/1/metrics',
+              metrics_monitoring_url: '/root/acets-review-apps/environments/15/metrics',
+              external_url: 'http://diplo.',
+              external_url_formatted: 'diplo.',
+              deployed_at: '2017-03-22T22:44:42.258Z',
+              deployed_at_formatted: 'Mar 22, 2017 10:44pm',
+              changes: [
+                {
+                  path: 'index.html',
+                  external_url: 'http://root-master-patch-91341.volatile-watch.surge.sh/index.html',
+                },
+                {
+                  path: 'imgs/gallery.html',
+                  external_url:
+                    'http://root-master-patch-91341.volatile-watch.surge.sh/imgs/gallery.html',
+                },
+                {
+                  path: 'about/',
+                  external_url: 'http://root-master-patch-91341.volatile-watch.surge.sh/about/',
+                },
+              ],
+              status: 'success',
+            },
+          ];
+
+          vm.$nextTick(done);
+        });
+
+        it('renders post deployment information', () => {
+          expect(vm.$el.querySelector('.js-post-deployment')).not.toBeNull();
+        });
+      });
+    });
+
+    describe('without information for target branch pipeline', () => {
+      beforeEach(done => {
+        vm.mr.state = 'merged';
+
+        vm.$nextTick(done);
+      });
+
+      it('does not render pipeline block', () => {
+        expect(vm.$el.querySelector('.js-post-merge-pipeline')).toBeNull();
+      });
+    });
+
+    describe('when state is not merged', () => {
+      beforeEach(done => {
+        vm.mr.state = 'archived';
+
+        vm.$nextTick(done);
+      });
+
+      it('does not render pipeline block', () => {
+        expect(vm.$el.querySelector('.js-post-merge-pipeline')).toBeNull();
+      });
+
+      it('does not render post deployment information', () => {
+        expect(vm.$el.querySelector('.js-post-deployment')).toBeNull();
+      });
+    });
   });
 });
