@@ -41,6 +41,52 @@ describe Gitlab::Diff::File do
     end
   end
 
+  describe '#unfold_diff_lines' do
+    let(:unfolded_lines) { double('expanded-lines') }
+    let(:unfolder) { instance_double(Gitlab::Diff::LinesUnfolder) }
+    let(:position) { instance_double(Gitlab::Diff::Position, old_line: 10) }
+
+    before do
+      allow(Gitlab::Diff::LinesUnfolder).to receive(:new) { unfolder }
+    end
+
+    context 'when unfold required' do
+      before do
+        allow(unfolder).to receive(:unfold_required?) { true }
+        allow(unfolder).to receive(:unfolded_diff_lines) { unfolded_lines }
+      end
+
+      it 'changes @unfolded to true' do
+        diff_file.unfold_diff_lines(position)
+
+        expect(diff_file).to be_unfolded
+      end
+
+      it 'updates @diff_lines' do
+        diff_file.unfold_diff_lines(position)
+
+        expect(diff_file.diff_lines).to eq(unfolded_lines)
+      end
+    end
+
+    context 'when unfold not required' do
+      before do
+        allow(unfolder).to receive(:unfold_required?) { false }
+      end
+
+      it 'keeps @unfolded false' do
+        diff_file.unfold_diff_lines(position)
+
+        expect(diff_file).not_to be_unfolded
+      end
+
+      it 'does not update @diff_lines' do
+        expect { diff_file.unfold_diff_lines(position) }
+          .not_to change(diff_file, :diff_lines)
+      end
+    end
+  end
+
   describe '#mode_changed?' do
     it { expect(diff_file.mode_changed?).to be_falsey }
   end
@@ -264,7 +310,7 @@ describe Gitlab::Diff::File do
     context 'when the content changed' do
       context 'when the file represented by the diff file is binary' do
         before do
-          allow(diff_file).to receive(:raw_binary?).and_return(true)
+          allow(diff_file).to receive(:binary?).and_return(true)
         end
 
         it 'returns a No Preview viewer' do
@@ -299,7 +345,7 @@ describe Gitlab::Diff::File do
 
       context 'when the file represented by the diff file is binary' do
         before do
-          allow(diff_file).to receive(:raw_binary?).and_return(true)
+          allow(diff_file).to receive(:binary?).and_return(true)
         end
 
         it 'returns an Added viewer' do
@@ -334,7 +380,7 @@ describe Gitlab::Diff::File do
 
       context 'when the file represented by the diff file is binary' do
         before do
-          allow(diff_file).to receive(:raw_binary?).and_return(true)
+          allow(diff_file).to receive(:binary?).and_return(true)
         end
 
         it 'returns a Deleted viewer' do
@@ -390,7 +436,7 @@ describe Gitlab::Diff::File do
         allow(diff_file).to receive(:deleted_file?).and_return(false)
         allow(diff_file).to receive(:renamed_file?).and_return(false)
         allow(diff_file).to receive(:mode_changed?).and_return(false)
-        allow(diff_file).to receive(:raw_text?).and_return(false)
+        allow(diff_file).to receive(:text?).and_return(false)
       end
 
       it 'returns a No Preview viewer' do
@@ -537,6 +583,12 @@ describe Gitlab::Diff::File do
       end
     end
 
+    describe '#empty?' do
+      it 'returns true' do
+        expect(diff_file.empty?).to be_truthy
+      end
+    end
+
     describe '#different_type?' do
       it 'returns false' do
         expect(diff_file).not_to be_different_type
@@ -616,4 +668,87 @@ describe Gitlab::Diff::File do
       end
     end
   end
+
+  describe '#empty?' do
+    let(:project) do
+      create(:project, :custom_repo, files: {})
+    end
+    let(:branch_name) { 'master' }
+
+    def create_file(file_name, content)
+      Files::CreateService.new(
+        project,
+        project.owner,
+        commit_message: 'Update',
+        start_branch: branch_name,
+        branch_name: branch_name,
+        file_path: file_name,
+        file_content: content
+      ).execute
+
+      project.commit(branch_name).diffs.diff_files.first
+    end
+
+    def update_file(file_name, content)
+      Files::UpdateService.new(
+        project,
+        project.owner,
+        commit_message: 'Update',
+        start_branch: branch_name,
+        branch_name: branch_name,
+        file_path: file_name,
+        file_content: content
+      ).execute
+
+      project.commit(branch_name).diffs.diff_files.first
+    end
+
+    def delete_file(file_name)
+      Files::DeleteService.new(
+        project,
+        project.owner,
+        commit_message: 'Update',
+        start_branch: branch_name,
+        branch_name: branch_name,
+        file_path: file_name
+      ).execute
+
+      project.commit(branch_name).diffs.diff_files.first
+    end
+
+    context 'when empty file is created' do
+      it 'returns true' do
+        diff_file = create_file('empty.md', '')
+
+        expect(diff_file.empty?).to be_truthy
+      end
+    end
+
+    context 'when empty file is deleted' do
+      it 'returns true' do
+        create_file('empty.md', '')
+        diff_file = delete_file('empty.md')
+
+        expect(diff_file.empty?).to be_truthy
+      end
+    end
+
+    context 'when file with content is truncated' do
+      it 'returns false' do
+        create_file('with-content.md', 'file content')
+        diff_file = update_file('with-content.md', '')
+
+        expect(diff_file.empty?).to be_falsey
+      end
+    end
+
+    context 'when empty file has content added' do
+      it 'returns false' do
+        create_file('empty.md', '')
+        diff_file = update_file('empty.md', 'new content')
+
+        expect(diff_file.empty?).to be_falsey
+      end
+    end
+  end
 end
