@@ -30,7 +30,7 @@ describe Repository do
 
   def expect_to_raise_storage_error
     expect { yield }.to raise_error do |exception|
-      storage_exceptions = [Gitlab::Git::Storage::Inaccessible, Gitlab::Git::CommandError, GRPC::Unavailable]
+      storage_exceptions = [Gitlab::Git::CommandError, GRPC::Unavailable]
       known_exception = storage_exceptions.select { |e| exception.is_a?(e) }
 
       expect(known_exception).not_to be_nil
@@ -38,49 +38,29 @@ describe Repository do
   end
 
   describe '#branch_names_contains' do
-    shared_examples '#branch_names_contains' do
-      set(:project) { create(:project, :repository) }
-      let(:repository) { project.repository }
+    set(:project) { create(:project, :repository) }
+    let(:repository) { project.repository }
 
-      subject { repository.branch_names_contains(sample_commit.id) }
+    subject { repository.branch_names_contains(sample_commit.id) }
 
-      it { is_expected.to include('master') }
-      it { is_expected.not_to include('feature') }
-      it { is_expected.not_to include('fix') }
+    it { is_expected.to include('master') }
+    it { is_expected.not_to include('feature') }
+    it { is_expected.not_to include('fix') }
 
-      describe 'when storage is broken', :broken_storage do
-        it 'should raise a storage error' do
-          expect_to_raise_storage_error do
-            broken_repository.branch_names_contains(sample_commit.id)
-          end
+    describe 'when storage is broken', :broken_storage do
+      it 'should raise a storage error' do
+        expect_to_raise_storage_error do
+          broken_repository.branch_names_contains(sample_commit.id)
         end
       end
     end
-
-    context 'when gitaly is enabled' do
-      it_behaves_like '#branch_names_contains'
-    end
-
-    context 'when gitaly is disabled', :skip_gitaly_mock do
-      it_behaves_like '#branch_names_contains'
-    end
   end
 
   describe '#tag_names_contains' do
-    shared_examples '#tag_names_contains' do
-      subject { repository.tag_names_contains(sample_commit.id) }
+    subject { repository.tag_names_contains(sample_commit.id) }
 
-      it { is_expected.to include('v1.1.0') }
-      it { is_expected.not_to include('v1.0.0') }
-    end
-
-    context 'when gitaly is enabled' do
-      it_behaves_like '#tag_names_contains'
-    end
-
-    context 'when gitaly is enabled', :skip_gitaly_mock do
-      it_behaves_like '#tag_names_contains'
-    end
+    it { is_expected.to include('v1.1.0') }
+    it { is_expected.not_to include('v1.0.0') }
   end
 
   describe 'tags_sorted_by' do
@@ -238,61 +218,41 @@ describe Repository do
   end
 
   describe '#last_commit_for_path' do
-    shared_examples 'getting last commit for path' do
-      subject { repository.last_commit_for_path(sample_commit.id, '.gitignore').id }
+    subject { repository.last_commit_for_path(sample_commit.id, '.gitignore').id }
 
-      it { is_expected.to eq('c1acaa58bbcbc3eafe538cb8274ba387047b69f8') }
+    it { is_expected.to eq('c1acaa58bbcbc3eafe538cb8274ba387047b69f8') }
 
-      describe 'when storage is broken', :broken_storage do
-        it 'should raise a storage error' do
-          expect_to_raise_storage_error do
-            broken_repository.last_commit_id_for_path(sample_commit.id, '.gitignore')
-          end
+    describe 'when storage is broken', :broken_storage do
+      it 'should raise a storage error' do
+        expect_to_raise_storage_error do
+          broken_repository.last_commit_id_for_path(sample_commit.id, '.gitignore')
         end
       end
     end
-
-    context 'when Gitaly feature last_commit_for_path is enabled' do
-      it_behaves_like 'getting last commit for path'
-    end
-
-    context 'when Gitaly feature last_commit_for_path is disabled', :skip_gitaly_mock do
-      it_behaves_like 'getting last commit for path'
-    end
   end
 
   describe '#last_commit_id_for_path' do
-    shared_examples 'getting last commit ID for path' do
-      subject { repository.last_commit_id_for_path(sample_commit.id, '.gitignore') }
+    subject { repository.last_commit_id_for_path(sample_commit.id, '.gitignore') }
 
-      it "returns last commit id for a given path" do
-        is_expected.to eq('c1acaa58bbcbc3eafe538cb8274ba387047b69f8')
-      end
+    it "returns last commit id for a given path" do
+      is_expected.to eq('c1acaa58bbcbc3eafe538cb8274ba387047b69f8')
+    end
 
-      it "caches last commit id for a given path" do
-        cache = repository.send(:cache)
-        key = "last_commit_id_for_path:#{sample_commit.id}:#{Digest::SHA1.hexdigest('.gitignore')}"
+    it "caches last commit id for a given path" do
+      cache = repository.send(:cache)
+      key = "last_commit_id_for_path:#{sample_commit.id}:#{Digest::SHA1.hexdigest('.gitignore')}"
 
-        expect(cache).to receive(:fetch).with(key).and_return('c1acaa5')
-        is_expected.to eq('c1acaa5')
-      end
+      expect(cache).to receive(:fetch).with(key).and_return('c1acaa5')
+      is_expected.to eq('c1acaa5')
+    end
 
-      describe 'when storage is broken', :broken_storage do
-        it 'should raise a storage error' do
-          expect_to_raise_storage_error do
-            broken_repository.last_commit_for_path(sample_commit.id, '.gitignore').id
-          end
+    describe 'when storage is broken', :broken_storage do
+      it 'should raise a storage error' do
+        expect_to_raise_storage_error do
+          broken_repository.last_commit_for_path(sample_commit.id, '.gitignore').id
         end
       end
     end
-
-    context 'when Gitaly feature last_commit_for_path is enabled' do
-      it_behaves_like 'getting last commit ID for path'
-    end
-
-    context 'when Gitaly feature last_commit_for_path is disabled', :skip_gitaly_mock do
-      it_behaves_like 'getting last commit ID for path'
-    end
   end
 
   describe '#commits' do
@@ -374,78 +334,57 @@ describe Repository do
 
   describe '#commits_by' do
     set(:project) { create(:project, :repository) }
+    let(:oids) { TestEnv::BRANCH_SHA.values }
 
-    shared_examples 'batch commits fetching' do
-      let(:oids) { TestEnv::BRANCH_SHA.values }
+    subject { project.repository.commits_by(oids: oids) }
 
-      subject { project.repository.commits_by(oids: oids) }
+    it 'finds each commit' do
+      expect(subject).not_to include(nil)
+      expect(subject.size).to eq(oids.size)
+    end
 
-      it 'finds each commit' do
+    it 'returns only Commit instances' do
+      expect(subject).to all( be_a(Commit) )
+    end
+
+    context 'when some commits are not found ' do
+      let(:oids) do
+        ['deadbeef'] + TestEnv::BRANCH_SHA.values.first(10)
+      end
+
+      it 'returns only found commits' do
         expect(subject).not_to include(nil)
-        expect(subject.size).to eq(oids.size)
-      end
-
-      it 'returns only Commit instances' do
-        expect(subject).to all( be_a(Commit) )
-      end
-
-      context 'when some commits are not found ' do
-        let(:oids) do
-          ['deadbeef'] + TestEnv::BRANCH_SHA.values.first(10)
-        end
-
-        it 'returns only found commits' do
-          expect(subject).not_to include(nil)
-          expect(subject.size).to eq(10)
-        end
-      end
-
-      context 'when no oids are passed' do
-        let(:oids) { [] }
-
-        it 'does not call #batch_by_oid' do
-          expect(Gitlab::Git::Commit).not_to receive(:batch_by_oid)
-
-          subject
-        end
+        expect(subject.size).to eq(10)
       end
     end
 
-    context 'when Gitaly list_commits_by_oid is enabled' do
-      it_behaves_like 'batch commits fetching'
-    end
+    context 'when no oids are passed' do
+      let(:oids) { [] }
 
-    context 'when Gitaly list_commits_by_oid is enabled', :disable_gitaly do
-      it_behaves_like 'batch commits fetching'
+      it 'does not call #batch_by_oid' do
+        expect(Gitlab::Git::Commit).not_to receive(:batch_by_oid)
+
+        subject
+      end
     end
   end
 
   describe '#find_commits_by_message' do
-    shared_examples 'finding commits by message' do
-      it 'returns commits with messages containing a given string' do
-        commit_ids = repository.find_commits_by_message('submodule').map(&:id)
+    it 'returns commits with messages containing a given string' do
+      commit_ids = repository.find_commits_by_message('submodule').map(&:id)
 
-        expect(commit_ids).to include(
-          '5937ac0a7beb003549fc5fd26fc247adbce4a52e',
-          '6f6d7e7ed97bb5f0054f2b1df789b39ca89b6ff9',
-          'cfe32cf61b73a0d5e9f13e774abde7ff789b1660'
-        )
-        expect(commit_ids).not_to include('913c66a37b4a45b9769037c55c2d238bd0942d2e')
-      end
-
-      it 'is case insensitive' do
-        commit_ids = repository.find_commits_by_message('SUBMODULE').map(&:id)
-
-        expect(commit_ids).to include('5937ac0a7beb003549fc5fd26fc247adbce4a52e')
-      end
+      expect(commit_ids).to include(
+        '5937ac0a7beb003549fc5fd26fc247adbce4a52e',
+        '6f6d7e7ed97bb5f0054f2b1df789b39ca89b6ff9',
+        'cfe32cf61b73a0d5e9f13e774abde7ff789b1660'
+      )
+      expect(commit_ids).not_to include('913c66a37b4a45b9769037c55c2d238bd0942d2e')
     end
 
-    context 'when Gitaly commits_by_message feature is enabled' do
-      it_behaves_like 'finding commits by message'
-    end
+    it 'is case insensitive' do
+      commit_ids = repository.find_commits_by_message('SUBMODULE').map(&:id)
 
-    context 'when Gitaly commits_by_message feature is disabled', :skip_gitaly_mock do
-      it_behaves_like 'finding commits by message'
+      expect(commit_ids).to include('5937ac0a7beb003549fc5fd26fc247adbce4a52e')
     end
 
     describe 'when storage is broken', :broken_storage do
@@ -461,6 +400,24 @@ describe Repository do
 
       it { is_expected.to be_nil }
     end
+
+    context 'regular blob' do
+      subject { repository.blob_at(repository.head_commit.sha, '.gitignore') }
+
+      it { is_expected.to be_an_instance_of(::Blob) }
+    end
+
+    context 'readme blob on HEAD' do
+      subject { repository.blob_at(repository.head_commit.sha, 'README.md') }
+
+      it { is_expected.to be_an_instance_of(::ReadmeBlob) }
+    end
+
+    context 'readme blob not on HEAD' do
+      subject { repository.blob_at(repository.find_branch('feature').target, 'README.md') }
+
+      it { is_expected.to be_an_instance_of(::Blob) }
+    end
   end
 
   describe '#merged_to_root_ref?' do
@@ -1048,6 +1005,67 @@ describe Repository do
     end
   end
 
+  describe '#ambiguous_ref?' do
+    let(:ref) { 'ref' }
+
+    subject { repository.ambiguous_ref?(ref) }
+
+    context 'when ref is ambiguous' do
+      before do
+        repository.add_tag(project.creator, ref, 'master')
+        repository.add_branch(project.creator, ref, 'master')
+      end
+
+      it 'should be true' do
+        is_expected.to eq(true)
+      end
+    end
+
+    context 'when ref is not ambiguous' do
+      before do
+        repository.add_tag(project.creator, ref, 'master')
+      end
+
+      it 'should be false' do
+        is_expected.to eq(false)
+      end
+    end
+  end
+
+  describe '#expand_ref' do
+    let(:ref) { 'ref' }
+
+    subject { repository.expand_ref(ref) }
+
+    context 'when ref is not tag or branch name' do
+      let(:ref) { 'refs/heads/master' }
+
+      it 'returns nil' do
+        is_expected.to eq(nil)
+      end
+    end
+
+    context 'when ref is tag name' do
+      before do
+        repository.add_tag(project.creator, ref, 'master')
+      end
+
+      it 'returns the tag ref' do
+        is_expected.to eq("refs/tags/#{ref}")
+      end
+    end
+
+    context 'when ref is branch name' do
+      before do
+        repository.add_branch(project.creator, ref, 'master')
+      end
+
+      it 'returns the branch ref' do
+        is_expected.to eq("refs/heads/#{ref}")
+      end
+    end
+  end
+
   describe '#add_branch' do
     let(:branch_name) { 'new_feature' }
     let(:target) { 'master' }
@@ -1219,6 +1237,27 @@ describe Repository do
     end
   end
 
+  describe '#blobs_at' do
+    let(:empty_repository) { create(:project_empty_repo).repository }
+
+    it 'returns empty array for an empty repository' do
+      # rubocop:disable Style/WordArray
+      expect(empty_repository.blobs_at(['master', 'foobar'])).to eq([])
+      # rubocop:enable Style/WordArray
+    end
+
+    it 'returns blob array for a non-empty repository' do
+      repository.create_file(User.last, 'foobar', 'CONTENT', message: 'message', branch_name: 'master')
+
+      # rubocop:disable Style/WordArray
+      blobs = repository.blobs_at([['master', 'foobar']])
+      # rubocop:enable Style/WordArray
+
+      expect(blobs.first.name).to eq('foobar')
+      expect(blobs.size).to eq(1)
+    end
+  end
+
   describe '#root_ref' do
     it 'returns a branch name' do
       expect(repository.root_ref).to be_an_instance_of(String)
@@ -1310,34 +1349,23 @@ describe Repository do
 
   describe '#merge' do
     let(:merge_request) { create(:merge_request, source_branch: 'feature', target_branch: 'master', source_project: project) }
-
     let(:message) { 'Test \r\n\r\n message' }
 
-    shared_examples '#merge' do
-      it 'merges the code and returns the commit id' do
-        expect(merge_commit).to be_present
-        expect(repository.blob_at(merge_commit.id, 'files/ruby/feature.rb')).to be_present
-      end
-
-      it 'sets the `in_progress_merge_commit_sha` flag for the given merge request' do
-        merge_commit_id = merge(repository, user, merge_request, message)
-
-        expect(merge_request.in_progress_merge_commit_sha).to eq(merge_commit_id)
-      end
-
-      it 'removes carriage returns from commit message' do
-        merge_commit_id = merge(repository, user, merge_request, message)
-
-        expect(repository.commit(merge_commit_id).message).to eq(message.delete("\r"))
-      end
+    it 'merges the code and returns the commit id' do
+      expect(merge_commit).to be_present
+      expect(repository.blob_at(merge_commit.id, 'files/ruby/feature.rb')).to be_present
     end
 
-    context 'with gitaly' do
-      it_behaves_like '#merge'
+    it 'sets the `in_progress_merge_commit_sha` flag for the given merge request' do
+      merge_commit_id = merge(repository, user, merge_request, message)
+
+      expect(merge_request.in_progress_merge_commit_sha).to eq(merge_commit_id)
     end
 
-    context 'without gitaly', :skip_gitaly_mock do
-      it_behaves_like '#merge'
+    it 'removes carriage returns from commit message' do
+      merge_commit_id = merge(repository, user, merge_request, message)
+
+      expect(repository.commit(merge_commit_id).message).to eq(message.delete("\r"))
     end
 
     def merge(repository, user, merge_request, message)
@@ -1374,97 +1402,77 @@ describe Repository do
   end
 
   describe '#revert' do
-    shared_examples 'reverting a commit' do
-      let(:new_image_commit) { repository.commit('33f3729a45c02fc67d00adb1b8bca394b0e761d9') }
-      let(:update_image_commit) { repository.commit('2f63565e7aac07bcdadb654e253078b727143ec4') }
-      let(:message) { 'revert message' }
+    let(:new_image_commit) { repository.commit('33f3729a45c02fc67d00adb1b8bca394b0e761d9') }
+    let(:update_image_commit) { repository.commit('2f63565e7aac07bcdadb654e253078b727143ec4') }
+    let(:message) { 'revert message' }
 
-      context 'when there is a conflict' do
-        it 'raises an error' do
-          expect { repository.revert(user, new_image_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
-        end
-      end
-
-      context 'when commit was already reverted' do
-        it 'raises an error' do
-          repository.revert(user, update_image_commit, 'master', message)
-
-          expect { repository.revert(user, update_image_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
-        end
-      end
-
-      context 'when commit can be reverted' do
-        it 'reverts the changes' do
-          expect(repository.revert(user, update_image_commit, 'master', message)).to be_truthy
-        end
-      end
-
-      context 'reverting a merge commit' do
-        it 'reverts the changes' do
-          merge_commit
-          expect(repository.blob_at_branch('master', 'files/ruby/feature.rb')).to be_present
-
-          repository.revert(user, merge_commit, 'master', message)
-          expect(repository.blob_at_branch('master', 'files/ruby/feature.rb')).not_to be_present
-        end
+    context 'when there is a conflict' do
+      it 'raises an error' do
+        expect { repository.revert(user, new_image_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
       end
     end
 
-    context 'when Gitaly revert feature is enabled' do
-      it_behaves_like 'reverting a commit'
+    context 'when commit was already reverted' do
+      it 'raises an error' do
+        repository.revert(user, update_image_commit, 'master', message)
+
+        expect { repository.revert(user, update_image_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
+      end
     end
 
-    context 'when Gitaly revert feature is disabled', :disable_gitaly do
-      it_behaves_like 'reverting a commit'
+    context 'when commit can be reverted' do
+      it 'reverts the changes' do
+        expect(repository.revert(user, update_image_commit, 'master', message)).to be_truthy
+      end
+    end
+
+    context 'reverting a merge commit' do
+      it 'reverts the changes' do
+        merge_commit
+        expect(repository.blob_at_branch('master', 'files/ruby/feature.rb')).to be_present
+
+        repository.revert(user, merge_commit, 'master', message)
+        expect(repository.blob_at_branch('master', 'files/ruby/feature.rb')).not_to be_present
+      end
     end
   end
 
   describe '#cherry_pick' do
-    shared_examples 'cherry-picking a commit' do
-      let(:conflict_commit) { repository.commit('c642fe9b8b9f28f9225d7ea953fe14e74748d53b') }
-      let(:pickable_commit) { repository.commit('7d3b0f7cff5f37573aea97cebfd5692ea1689924') }
-      let(:pickable_merge) { repository.commit('e56497bb5f03a90a51293fc6d516788730953899') }
-      let(:message) { 'cherry-pick message' }
+    let(:conflict_commit) { repository.commit('c642fe9b8b9f28f9225d7ea953fe14e74748d53b') }
+    let(:pickable_commit) { repository.commit('7d3b0f7cff5f37573aea97cebfd5692ea1689924') }
+    let(:pickable_merge) { repository.commit('e56497bb5f03a90a51293fc6d516788730953899') }
+    let(:message) { 'cherry-pick message' }
 
-      context 'when there is a conflict' do
-        it 'raises an error' do
-          expect { repository.cherry_pick(user, conflict_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
-        end
-      end
-
-      context 'when commit was already cherry-picked' do
-        it 'raises an error' do
-          repository.cherry_pick(user, pickable_commit, 'master', message)
-
-          expect { repository.cherry_pick(user, pickable_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
-        end
-      end
-
-      context 'when commit can be cherry-picked' do
-        it 'cherry-picks the changes' do
-          expect(repository.cherry_pick(user, pickable_commit, 'master', message)).to be_truthy
-        end
-      end
-
-      context 'cherry-picking a merge commit' do
-        it 'cherry-picks the changes' do
-          expect(repository.blob_at_branch('improve/awesome', 'foo/bar/.gitkeep')).to be_nil
-
-          cherry_pick_commit_sha = repository.cherry_pick(user, pickable_merge, 'improve/awesome', message)
-          cherry_pick_commit_message = project.commit(cherry_pick_commit_sha).message
-
-          expect(repository.blob_at_branch('improve/awesome', 'foo/bar/.gitkeep')).not_to be_nil
-          expect(cherry_pick_commit_message).to eq(message)
-        end
+    context 'when there is a conflict' do
+      it 'raises an error' do
+        expect { repository.cherry_pick(user, conflict_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
       end
     end
 
-    context 'when Gitaly cherry_pick feature is enabled' do
-      it_behaves_like 'cherry-picking a commit'
+    context 'when commit was already cherry-picked' do
+      it 'raises an error' do
+        repository.cherry_pick(user, pickable_commit, 'master', message)
+
+        expect { repository.cherry_pick(user, pickable_commit, 'master', message) }.to raise_error(Gitlab::Git::Repository::CreateTreeError)
+      end
     end
 
-    context 'when Gitaly cherry_pick feature is disabled', :disable_gitaly do
-      it_behaves_like 'cherry-picking a commit'
+    context 'when commit can be cherry-picked' do
+      it 'cherry-picks the changes' do
+        expect(repository.cherry_pick(user, pickable_commit, 'master', message)).to be_truthy
+      end
+    end
+
+    context 'cherry-picking a merge commit' do
+      it 'cherry-picks the changes' do
+        expect(repository.blob_at_branch('improve/awesome', 'foo/bar/.gitkeep')).to be_nil
+
+        cherry_pick_commit_sha = repository.cherry_pick(user, pickable_merge, 'improve/awesome', message)
+        cherry_pick_commit_message = project.commit(cherry_pick_commit_sha).message
+
+        expect(repository.blob_at_branch('improve/awesome', 'foo/bar/.gitkeep')).not_to be_nil
+        expect(cherry_pick_commit_message).to eq(message)
+      end
     end
   end
 
@@ -1562,12 +1570,12 @@ describe Repository do
         :size,
         :commit_count,
         :rendered_readme,
+        :readme_path,
         :contribution_guide,
         :changelog,
         :license_blob,
         :license_key,
         :gitignore,
-        :koding_yml,
         :gitlab_ci_yml,
         :branch_names,
         :tag_names,
@@ -1921,38 +1929,27 @@ describe Repository do
     end
   end
 
-  describe '#koding_yml', :use_clean_rails_memory_store_caching do
-    it 'returns and caches the output' do
-      expect(repository).to receive(:file_on_head)
-        .with(:koding)
-        .and_return(Gitlab::Git::Tree.new(path: '.koding.yml'))
-        .once
-
-      2.times do
-        expect(repository.koding_yml).to be_an_instance_of(Gitlab::Git::Tree)
-      end
-    end
-  end
-
   describe '#readme', :use_clean_rails_memory_store_caching do
     context 'with a non-existing repository' do
-      it 'returns nil' do
-        allow(repository).to receive(:tree).with(:head).and_return(nil)
+      let(:project) { create(:project) }
 
+      it 'returns nil' do
         expect(repository.readme).to be_nil
       end
     end
 
     context 'with an existing repository' do
       context 'when no README exists' do
-        it 'returns nil' do
-          allow_any_instance_of(Tree).to receive(:readme).and_return(nil)
+        let(:project) { create(:project, :empty_repo) }
 
+        it 'returns nil' do
           expect(repository.readme).to be_nil
         end
       end
 
       context 'when a README exists' do
+        let(:project) { create(:project, :repository) }
+
         it 'returns the README' do
           expect(repository.readme).to be_an_instance_of(ReadmeBlob)
         end
@@ -1960,6 +1957,42 @@ describe Repository do
     end
   end
 
+  describe '#readme_path', :use_clean_rails_memory_store_caching do
+    context 'with a non-existing repository' do
+      let(:project) { create(:project) }
+
+      it 'returns nil' do
+        expect(repository.readme_path).to be_nil
+      end
+    end
+
+    context 'with an existing repository' do
+      context 'when no README exists' do
+        let(:project) { create(:project, :empty_repo) }
+
+        it 'returns nil' do
+          expect(repository.readme_path).to be_nil
+        end
+      end
+
+      context 'when a README exists' do
+        let(:project) { create(:project, :repository) }
+
+        it 'returns the README' do
+          expect(repository.readme_path).to eq("README.md")
+        end
+
+        it 'caches the response' do
+          expect(repository).to receive(:readme).and_call_original.once
+
+          2.times do
+            expect(repository.readme_path).to eq("README.md")
+          end
+        end
+      end
+    end
+  end
+
   describe '#expire_statistics_caches' do
     it 'expires the caches' do
       expect(repository).to receive(:expire_method_caches)
@@ -2128,9 +2161,10 @@ describe Repository do
   describe '#refresh_method_caches' do
     it 'refreshes the caches of the given types' do
       expect(repository).to receive(:expire_method_caches)
-        .with(%i(rendered_readme license_blob license_key license))
+        .with(%i(rendered_readme readme_path license_blob license_key license))
 
       expect(repository).to receive(:rendered_readme)
+      expect(repository).to receive(:readme_path)
       expect(repository).to receive(:license_blob)
       expect(repository).to receive(:license_key)
       expect(repository).to receive(:license)
@@ -2180,38 +2214,42 @@ describe Repository do
     rugged.references.create("refs/remotes/#{remote_name}/#{branch_name}", target.id)
   end
 
-  describe '#ancestor?' do
+  shared_examples '#ancestor?' do
     let(:commit) { repository.commit }
     let(:ancestor) { commit.parents.first }
 
-    shared_examples '#ancestor?' do
-      it 'it is an ancestor' do
-        expect(repository.ancestor?(ancestor.id, commit.id)).to eq(true)
-      end
-
-      it 'it is not an ancestor' do
-        expect(repository.ancestor?(commit.id, ancestor.id)).to eq(false)
-      end
-
-      it 'returns false on nil-values' do
-        expect(repository.ancestor?(nil, commit.id)).to eq(false)
-        expect(repository.ancestor?(ancestor.id, nil)).to eq(false)
-        expect(repository.ancestor?(nil, nil)).to eq(false)
-      end
-
-      it 'returns false for invalid commit IDs' do
-        expect(repository.ancestor?(commit.id, Gitlab::Git::BLANK_SHA)).to eq(false)
-        expect(repository.ancestor?( Gitlab::Git::BLANK_SHA, commit.id)).to eq(false)
-      end
+    it 'it is an ancestor' do
+      expect(repository.ancestor?(ancestor.id, commit.id)).to eq(true)
     end
 
-    context 'with Gitaly enabled' do
-      it_behaves_like('#ancestor?')
+    it 'it is not an ancestor' do
+      expect(repository.ancestor?(commit.id, ancestor.id)).to eq(false)
     end
 
-    context 'with Gitaly disabled', :skip_gitaly_mock do
-      it_behaves_like('#ancestor?')
+    it 'returns false on nil-values' do
+      expect(repository.ancestor?(nil, commit.id)).to eq(false)
+      expect(repository.ancestor?(ancestor.id, nil)).to eq(false)
+      expect(repository.ancestor?(nil, nil)).to eq(false)
     end
+
+    it 'returns false for invalid commit IDs' do
+      expect(repository.ancestor?(commit.id, Gitlab::Git::BLANK_SHA)).to eq(false)
+      expect(repository.ancestor?( Gitlab::Git::BLANK_SHA, commit.id)).to eq(false)
+    end
+  end
+
+  describe '#ancestor? with Gitaly enabled' do
+    it_behaves_like "#ancestor?"
+  end
+
+  describe '#ancestor? with Rugged enabled', :enable_rugged do
+    it 'calls out to the Rugged implementation' do
+      allow_any_instance_of(Rugged).to receive(:merge_base).with(repository.commit.id, Gitlab::Git::BLANK_SHA).and_call_original
+
+      repository.ancestor?(repository.commit.id, Gitlab::Git::BLANK_SHA)
+    end
+
+    it_behaves_like '#ancestor?'
   end
 
   describe '#archive_metadata' do
@@ -2267,6 +2305,7 @@ describe Repository do
       expect(subject).to be_a(Gitlab::Git::Repository)
       expect(subject.relative_path).to eq(project.disk_path + '.git')
       expect(subject.gl_repository).to eq("project-#{project.id}")
+      expect(subject.gl_project_path).to eq(project.full_path)
     end
 
     context 'with a wiki repository' do
@@ -2276,6 +2315,7 @@ describe Repository do
         expect(subject).to be_a(Gitlab::Git::Repository)
         expect(subject.relative_path).to eq(project.disk_path + '.wiki.git')
         expect(subject.gl_repository).to eq("wiki-#{project.id}")
+        expect(subject.gl_project_path).to eq(project.full_path)
       end
     end
   end
@@ -2386,4 +2426,15 @@ describe Repository do
       end
     end
   end
+
+  describe '#merge_base' do
+    set(:project) { create(:project, :repository) }
+    subject(:repository) { project.repository }
+
+    it 'only makes one gitaly call' do
+      expect(Gitlab::GitalyClient).to receive(:call).once.and_call_original
+
+      repository.merge_base('master', 'fix')
+    end
+  end
 end
