@@ -1,8 +1,10 @@
 require 'spec_helper'
 
 describe Clusters::UpdateService do
+  include KubernetesHelpers
+
   describe '#execute' do
-    subject { described_class.new(cluster.project, cluster.user, params).execute(cluster) }
+    subject { described_class.new(cluster.user, params).execute(cluster) }
 
     let(:cluster) { create(:cluster, :project, :provided_by_user) }
 
@@ -34,6 +36,11 @@ describe Clusters::UpdateService do
           }
         end
 
+        before do
+          allow(ClusterConfigureWorker).to receive(:perform_async)
+          stub_kubeclient_get_namespace('https://kubernetes.example.com', namespace: 'my-namespace')
+        end
+
         it 'updates namespace' do
           is_expected.to eq(true)
           expect(cluster.platform.namespace).to eq('custom-namespace')
@@ -55,5 +62,32 @@ describe Clusters::UpdateService do
         expect(cluster.errors[:"platform_kubernetes.namespace"]).to be_present
       end
     end
+
+    context 'when cluster is provided by GCP' do
+      let(:cluster) { create(:cluster, :project, :provided_by_gcp) }
+
+      let(:params) do
+        {
+          name: 'my-new-name'
+        }
+      end
+
+      it 'does not change cluster name' do
+        is_expected.to eq(false)
+
+        cluster.reload
+        expect(cluster.name).to eq('test-cluster')
+      end
+
+      context 'when cluster is being created' do
+        let(:cluster) { create(:cluster, :providing_by_gcp) }
+
+        it 'rejects changes' do
+          is_expected.to eq(false)
+
+          expect(cluster.errors.full_messages).to include('cannot modify during creation')
+        end
+      end
+    end
   end
 end
