@@ -23,7 +23,7 @@ RSpec.configure do |config|
     DatabaseCleaner.clean_with(:deletion, cache_tables: false)
   end
 
-  config.before(:each) do
+  config.before do
     DatabaseCleaner.strategy = :transaction
   end
 
@@ -39,11 +39,11 @@ RSpec.configure do |config|
     DatabaseCleaner.strategy = :deletion, { cache_tables: false }
   end
 
-  config.before(:each) do
+  config.before do
     DatabaseCleaner.start
   end
 
-  config.append_after(:each) do
+  config.append_after do
     DatabaseCleaner.clean
   end
 end
