@@ -34,4 +34,13 @@ module EmailHelpers
   def find_email_for(user)
     ActionMailer::Base.deliveries.find { |d| d.to.include?(user.notification_email) }
   end
+
+  def have_referable_subject(referable, include_project: true, reply: false)
+    prefix = (include_project && referable.project ? "#{referable.project.name} | " : '').freeze
+    prefix = "Re: #{prefix}" if reply
+
+    suffix = "#{referable.title} (#{referable.to_reference})"
+
+    have_subject [prefix, suffix].compact.join
+  end
 end
