@@ -26,7 +26,7 @@ shared_examples 'update invalid issuable' do |klass|
     end
 
     it 'renders edit when format is html' do
-      put :update, params
+      put :update, params: params
 
       expect(response).to render_template(:edit)
       expect(assigns[:conflict]).to be_truthy
@@ -35,7 +35,7 @@ shared_examples 'update invalid issuable' do |klass|
     it 'renders json error message when format is json' do
       params[:format] = "json"
 
-      put :update, params
+      put :update, params: params
 
       expect(response.status).to eq(409)
       expect(JSON.parse(response.body)).to have_key('errors')
@@ -49,7 +49,7 @@ shared_examples 'update invalid issuable' do |klass|
     end
 
     it 'renders edit when merge request is invalid' do
-      put :update, params
+      put :update, params: params
 
       expect(response).to render_template(:edit)
     end
