@@ -716,7 +716,7 @@ describe ObjectStorage do
           end
 
           let!(:fog_file) do
-            fog_connection.directories.get('uploads').files.create(
+            fog_connection.directories.new(key: 'uploads').files.create(
               key: 'tmp/uploads/test/123123',
               body: 'content'
             )
